-- CLCProt 4.0.00.003 beta .03 for WoW Warlords of Draenor 6.0.3 (pre-patch)
-- by Adeya, player of Paladin DyeimiAdeya currently on Garona_US
--
--    The CLCProt 4.x files include "YOU MUST READ ME NOW" (.rtf), which has  
-- that name because it is true. :-) It is in the same folder as clcprot.lua.
--
--   CLCProt 4.x is the successor of "CLCProt for WoW Mists of Pandaria 5.4".
-- It is updated for WoW Warlords of Draenor 6.x, to include its new features 
-- and many changes which have been documented by Blizzard Entertainment and 
-- by fan websites such as Wowhead, Wowpedia, Maintankadin, and Elitist Jerks.
--
--   The algorithim uses the expiration time of spell cooldowns to select the
-- action with the highest priority that is usable, for "the primary skill" 
-- and "the secondary skill" respectively. When expiration time is zero, the 
-- spell is not on cooldown. However, if the expiration time is less than or 
-- equal to the expiration time of the current global cooldown (actual or pro- 
-- jected), then its expiration time is returned as zero since it will not be 
-- on cooldown when the current global cooldown ends. Using expiration times  
-- is more accurate and reliable than the time-remaining method used in prior 
-- versions. 
--
-- Further notes:
--
-- (1) Level 100 Talents: 
--
--   Seraphim is best managed as a raid cooldown since it costs 5 HP to cast; 
-- its duration and cooldown are (currently) not the same; it does not trigger 
-- a global cooldown.
--
--   Empowered Seals applies one of three buffs according to whether the Seal 
-- of Insight, Seal of Righteousness, or Seal of Truth is in effect when the 
-- player casts Judgment. Further investigation is necessary to implement it. 
-- Most likely, each buff will have its own ID, but no data has been found.
--
-- (2) Currently, GetSpellCooldown(61304) always returns 0 for both the start 
-- and the duration of Global Cooldown when called immediately after a spell 
-- that triggers a GCD has been cast. The only evidence that a GCD is in pro-
-- gress is the "radial sweep" that is shown on the texture of spell icons dis-
-- played in the CLCProt skill boxes.
--
--   Thus, a process for calculating a "GCD" primarily for use in determining 
-- whether a buff will expire before the end of the "current GCD" has been im-
-- plemented.
--
--   Prior comment: The duration of the -current- GDC is returned by GetSpell-
-- Cooldown(61304). Realize that the length of the GCD is not constant, but has 
-- a variable value that is calculated at the time the GCD is triggered. The 
-- duration is returned as zero when a GCD is not in progress. Perchance the 
-- GCD is returned as less than the minimum Paladin GCD of 1 second, but great-
-- er than zero, then the minimum is used instead.
-- 
-- (3) The API GetSpellCooldown() returns the starting time and the duration 
-- for the cooldown, which are zero if the spell is not on cooldown. The dura-
-- tion includes the effects of haste, buffs, debuffs, and procs which reduce 
-- it, so modification of its value is neither necessary nor desirable.	
--
--   The numeric countdown displayed by enabling the option to do so on the de- 
-- fault UI <ESC> System > System tab is the spell cooldown. It evidently does
-- disclose whether a spell shown in either or both of the skill boxes is actu-
-- ally off cooldown (albeit, the countdown is not always shown when it should 
-- be shown, in my observation). 
--
-- (4) In the GetCD() method: the condition  s1 == actionId  may be true while
-- parsing the priority sequence for the second skill (s2), and the action has 
-- has already been chosen as the primary skill (s1) during the same rotation. 
-- Thus a nominal expiration time is returned for that action instead. Actions 
-- are sorted into 3 groups according to priority: 
--
--   For the first group, the returned expiration is the current time plus 1.0  
-- sec. plus a priority time ranging from 0.11 to 1.49 sec.
-- 
--   For the second group, the returned expiration is the current time plus
-- 1.5 sec. plus a priority time ranging from 0.11 to 1.49 sec.
--
--  For all other usable actions the returned expiration time is the current 
-- value of the variable: later. If an action is not usable, then the expira-
-- tion time returned is the current value of the variable: muchlater.
--
--   The priority times for the first two groups can be configured on the Rota-
-- tion page Priority tab section Optional: Timing Settings (options.lua).

-- (5) UnitBuff() and UnitDebuff(), respectively, return the duration and ex-
-- piration time of the buff/debuff, without the starting time. I assume that 
-- the host's calculations included any effects of Haste, other buffs, debuffs, 
-- and/or procs which affected the duration of the specific buff/debuff.
--
-- (6) Spells which do not consume HP require a percentage of the character's 
-- maximum "base" mana instead. The configured minimum remaining mana in order 
-- to cast Consecration is a percentage of the maximum. A table containing the
-- mana cost of the most commonly-used spells is included in READ ME NOW.
--
-- (7) While the paladin has 4 of the 5 pieces in the armor Tier 16 set equip-
-- ped, the Bastion of Power (spell ID 144569) buff is applied when 3 or more 
-- BoG are stacked, signifying that a free cast for EF/WoG is available. The 
-- buff itself does not expire, but the free cast vanishes if either the BoG 
-- stack expires or the BoG stack is used.
--
-- PROVENANCE: CLCProt began as a module of clcInfo. The OOP source code was 
-- written in that context using the WoWAce toolkit and libraries, and subse-
-- quently modified to make CLCProt a standalone addon. The prior procedural 
-- source code for this continuation of CLCProt is from "CLCProt for WoW Mists 
-- of Pandaria 5.4" by Adeya, which was derived from "CLCProt 2.0 for MoP 5.4" 
-- by Blue Warlord. Some pieces of the source have been adopted from or model-
-- ed on source code in CLCRet by abija. The source code for creating the Rota-
-- tion tabs was modeled by Adeya from code in CLCRet for WoW 5.4. All source 
-- files were downloaded from Curse.com.
--
-- COURTESY NOTICE: the Lua scripting language is Copyright © 1994–2014 Lua.org, 
-- PUC-Rio. The Lua license is available at http://www.lua.org/license.html
-------------------------------------------------------------------------------=
---------------("------m-e-s-s-a-g-e---m-e-a-s-u-r-e-m-e-n-t----------------------------------------------------------------------------------")--

--[[
	-- chat command macros to get talent IDs for secondary protection spec and to verify them 
	
	/run talID = GetTalentInfo(3,1,2);print(talID)		-- Selfless Healer ID
	/run talID = GetTalentInfo(3,2,2);print(talID)	 	-- Eternal Flame ID 
	/run talID = GetTalentInfo(3,3,2);print(talID)		-- Sacred Shield ID
	/run talID = GetTalentInfo(3,3,1);print(talID)		-- Sacred Shield ID - Retribution spec

	/run talID = GetTalentInfo(5,2,2);print(talID)		-- Sanctified Wrath ID

	/run talID = GetTalentInfo(6,1,2);print(talID)		-- Holy Prism ID
	/run talID = GetTalentInfo(6,2,2);print(talID)	 	-- Light's Hammer ID
	/run talID = GetTalentInfo(6,3,2);print(talID)		-- Execution Sentence ID

	/run talID = GetTalentInfo(7,1,2);print(talID)		-- Empowered Seals ID
	/run talID = GetTalentInfo(7,2,2);print(talID)		-- Seraphim ID
	-------------------------------------------------------------------------------------
	/run _, name = GetTalentInfoByID(17581);print(name)	-- Selfless Healer
	/run _, name = GetTalentInfoByID(17583);print(name)	-- Eternal Flame
	/run _, name = GetTalentInfoByID(21811);print(name)	-- Sacred Shield
	/run _, name = GetTalentInfoByID(21098);print(name)	-- Sacred Shield - Retribution spec

	/run _, name = GetTalentInfoByID(21795);print(name)	-- Sanctified Wrath

	/run _, name = GetTalentInfoByID(17605);print(name)	-- Holy Prism
	/run _, name = GetTalentInfoByID(17607);print(name)	-- Light's Hammer
	/run _, name = GetTalentInfoByID(17609);print(name)	-- Execution Sentence


	/run _, name = GetTalentInfoByID(21201);print(name)	-- Empowered Seals
	/run _, name = GetTalentInfoByID(21202);print(name)	-- Seraphim
--]]

-- don't load if class is wrong
local _, class = UnitClass("player")
if class ~= "PALADIN" then return end						-- prior source, also in clcprot.lua (which is loaded first)

local GetTime = GetTime			-- prior source
local db = db					-- prior source 

-----------------------------------------------------------------------------------------------------------------------=
----: globals initialized in clcprot.lua:
--
-- s_protSpec is true if the paladin's spec is protection, else false; set near start of clcprot.lua
-- talentGroup = 0				-- whether protection spec is paladin's primary spec (1) or secondary spec (2)
--
-- talent and glyph flags
--  s_eft = false				-- true while T3 L45 Eternal Flame talent selected
--  s_swt = false				-- true while Sanctified Wrath is selected
--  s_goc = false				-- true while Glyph of Consecration is in effect
--
--  consId			= 26573	 	-- Consecration ID, depending upon whether GoC is in effect (no glyph is default)
----  consGlyphId	= 54928	 	-- Glyph of Consecration 
----  consIdGlyphed	= 116467 	-- consecration with gylph
----  consIdNoGlyph	= 26573  	-- consecration without glyph 	
--
-- s_queue = true 				-- true when either priority sequence has changed and must be validated
-- s_initPrio  = true			-- true until the initial priority sequence is displayed on ChatFrame1
-- useAltPrio  = false			-- true while the alternate priority sequence is the default
-- prioToggled = "no"			-- assigned "qn" or "qa" when default priority is changed (reset to "no")
--
-- useEFGalt   = false			-- true when player chooses the Glorious Eternal Flame alternative
-- useWoGalt   = false			-- true when player chooses the Word of Glory alternative
--    
-- s1_twm = false				-- true while the AoE of the spell in s1 must be chosen with a mouse 
-- s2_twm = false				-- true while the AoE of the spell in s2 must be chosen with a mouse 
--
--  rota = 0					-- tally of calls to clcprot.ProtRotation() since CLCProt was loaded (debugging)
--

-- list of spellId  ---------------------------------------------------------------------------------------------------=

local gcdId 				= 61304  -- Global Cooldown --dA: actual GCD "spell" 
local asId 					= 31935	 -- avenger's shield
local basId					= 114637 -- bastion of glory
local bopId					= 144569 -- bastion of power (armor Tier 16 4-piece set free cast for EF/WoG)
local csId 					= 35395	 -- crusader strike
local dpId					= 90174  -- divine purpose
local efId					= 114163 -- eternal flame
local esId 					= 114157 -- execution sentence
local gcId					= 85416  -- grand crusader
local haId					= 105809 -- holy avenger
local hotrId				= 53595  -- hammer of the righteous
local howId 				= 24275	 -- hammer of wrath
local hprId 				= 114165 -- holy prism		--dA: 114852 is wrong spell
local hwId					= 119072 -- holy wrath
local jId					= 20271	 -- judgment
local lhId 					= 114158 -- light's hammer
local sasId					= 20925  -- sacred shield				--dA: cf. 148039
local serId					= 152262 -- seraphim (6.0.2.00.01)
local sotrId					= 53600	 -- shield of the righteous
local swId					= 53376	 -- sanctified wrath
local wogId					= 85673  -- word of glory				--dA: cf. 136494

-- other variables local to the chunk  ---dA: initialized (many exceptions are caused by nils!) -----------------------=

-- the action priority sequence 
local qn = {} 			-- prior source, normal queue	
local qalt = {} 		-- prior sourcd, alt queue	
local q 				-- prior source, working queue

local s1, s2 = 0, 0				-- spell IDs for the primary & secondary skills (actions), respectively

local c_time = GetTime()		-- variable, time at the start of the current rotation; see GetStatus() 
local howlate = 90.000			-- constant, how many seconds to add to c_time for the value of later 
local later = 0.000				-- variable, expiration time for actions omitted from selection as s1 or s2 
local muchlater = 0.000			-- variable, more expiration time for some actions omitted from selection 

-- used by GetGlobalCooldown()
local gcdAcc = 0				-- variable, accumulated passing time since most recent GCD was triggered
local gcdNew = 0				-- variable, time that GCD was triggered - computed, not from GetSpellCooldown()
local gcdLen = 0				-- variable, length of the calculated GCD at time it is triggered
-- returned by GetGlobalCooldown()
local gcdDur = 0				-- variable, duration of current GCD - computed, not from GetSpellCooldown()
local gcdExp = 0.000			-- variable, expiration time of the current GCD - computed, not from GetSpellCooldown()
-- computed, based on gcdDur
local gcds2 = 0.000				-- variable, GCD duration increment to current time for UpdateStatus()

local c_bog = 0					-- variable, current count for the Bastion of Glory stack
local c_hp = 0					-- variable, current number of Holy Power charges
local c_mHaste = 0.00			-- variable, the paladin's calculated Melee Haste

local c_efHaO = 0				-- variable, current amt of EF heal-at-once (3.1.5.06)
local c_efHoT = 0				-- variable, current amt of EF heal-over-time per tick (3.1.5.06)
local c_wogHaO = 0				-- variable, current amt of WoG heal-at-once (3.1.5.06)

local s_bog = false				-- true while Bastion of Glory stack count > 0 (3.1.5.07)
local s_bop = false				-- true while Bastion of Power free cast for EF is available (3.1.5.07)
local s_dp = false				-- true while Divine Purpose free cast is available
local s_ef = false				-- true while Eternal Flame is in effect 
local s_gc = false				-- true while Grand Crusader is in effect
local s_ha = false				-- true while Holy Avenger is in effect
local s_ser = false				-- true while Seraphim is in effect (6.0.2.0.0.00)
local s_ss = false				-- true while Sacred Shield is active
local s_4t16 = false			-- true while 4-piece Tier 16 free cast for EF/WoG is available

local x_bog = 0.000				-- time that Bastion of Glory stack will expire
local x_dp = 0.000				-- time that Divine Purpose free cast proc will expire
local x_ef = 0.000				-- time that Eternal Flame buff will expire
local x_gc = 0.000				-- time that Grand Crusader proc will expire
local x_ha = 0.000				-- time that Holy Avenger buff will expire
local x_ser = 0.000				-- time that Seraphim buff will expire (6.0.2.0.0.01)
local x_ss = 0.000				-- time that Sacred Shield buff will expire

local r_bog = 0.000				-- time remaining for the BoG stack (count > 0)
local r_dp = 0.000				-- time remaining for Divine Purpose proc 
local r_ef = 0.000				-- time remaining for Eternal Flame
local r_ss = 0.000				-- time remaining for Sacred Shield

-- action priority sequence flags 
local s_efc      = false		-- true when action efc is in the priority sequence that is in use 
local s_efg 	 = false		-- true when action efg is in the priority sequence that is in use
local s_wog		 = false		-- true when action wog is in the priority sequence that is in use

-- debugging variables			-- all added, also useful for examining performance and relationships among buffs & spells 
local s_s1 = false				-- true when the skill to be chosen is the primary skill
local count_getCD = 0           -- tally of calls to GetCooldown() during the current session 
local count_getGCD = 0 			-- tally of calls to GetGlobalCooldown() during the current session 

-----------------------------------------------------------------------------------------------------------------------
-- evaluated during addon load ----------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------

-- buff names (constants)
local buffBG = GetSpellInfo(basId)		-- returns string "Bastion of Glory"
local buffBP = GetSpellInfo(bopId)		-- returns string "Bastion of Power"
local buffDP = GetSpellInfo(dpId)		-- returns string "Divine Purpose"
local buffEF = GetSpellInfo(efId)		-- returns string "Eternal Flame"
local buffGC = GetSpellInfo(gcId)		-- returns string "Grand Crusader"
local buffHA = GetSpellInfo(haId)		-- returns string "Holy Avenger"
local buffSS = GetSpellInfo(sasId)		-- returns string "Sacred Shield"
local buffSR = GetSpellInfo(serId)		-- returns string "Seraphim"
local buffSW = GetSpellInfo(swId)		-- returns string "Sanctified Wrath"

--[[
print("CLCProt[",rota,"]: buff name for ",basId," buffBG: ",buffBG)
print("CLCProt[",rota,"]: buff name for ",bopId," buffBP: ",buffBP)
print("CLCProt[",rota,"]: buff name for ",dpId," buffDP: ",buffDP)
print("CLCProt[",rota,"]: buff name for ",efId," buffEF: ",buffEF)
print("CLCProt[",rota,"]: buff name for ",gcId," buffGC: ",buffGC)
print("CLCProt[",rota,"]: buff name for ",haId," buffHA: ",buffHA)
print("CLCProt[",rota,"]: buff name for ",sasId," buffSS: ",buffSS)
print("CLCProt[",rota,"]: buff name for ",serId," buffSR: ",buffSR)
print("CLCProt[",rota,"]: buff name for ",swId," buffSW: ",buffSW)
--]]

-----------------------------------------------------------------------------------------------------------------------

local function GetTargetHealthPercent()						-- used for Holy Wrath with Glyph of Final Wrath
	local targetHealthMax, targetHealthPercentage = 0, 0
	targetHealthMax = UnitHealthMax("target") 				-- returns zero if player has no target
	if targetHealthMax > 0 then								
		targetHealthPercentage = UnitHealth("target") / targetHealthMax 
	end
	return targetHealthPercentage							-- zero if no target 
end

local function GetCooldown(ability,id)						-- fundamentally revised

	-- GetCooldown() returns the time that the spell cooldown expires; if zero, the spell is not on cooldown.
	-- also returns zero if the cooldown expires before or at the time that the current GCD expires.

	count_getCD = count_getCD + 1							-- debugging variable
	local wait, start, duration, enabled, cdx = 0, 0, 0, 0, 0

	if ability and type(ability) == "string" then					 
		if id and type(id) == "number" and id > 0 then 

			start, duration, enabled = GetSpellCooldown(id) 	-- returns 0 for start & duration while spell is -not- on cooldown
			if enabled and enabled == 0 then return later end	-- enabled = 0 while spell is active but pending CD (else it returns 1)
			
			if start and duration then 							-- returns nils for nonesuch spell ID 
				cdx = start + duration				 			-- cdx is the time that the spell cooldown expires
				if cdx <= gcdExp then 							-- gcdExp is the time that the current global cooldown expires
					return 0 
				else 
					return cdx
				end
			end
			print("CLCProt: [",rota,"] GetCooldown() nonesuch id on call: ",count_getCD," ability: ",ability," id: ",id," s1: ",s1," s2: ",s2)
			return later
		end
		print("CLCProt: [",rota,"] GetCooldown() invalid id on call: ",count_getCD," ability: ",ability," id: ",id," s1: ",s1," s2: ",s2)
		return later
	end
	print("CLCProt: [",rota,"] GetCooldown() invalid ability on call: ",count_getCD," ability: ",ability," id: ",id," s1: ",s1," s2: ",s2)
	return later
end

-----------------------------------------------------------------------------------------------------------------------
-- actions ------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------------

actions = {													-- global variable used in clcprot.lua function(s)

	as = {	
		id = asId,					
		GetCD = function()
			if ( s1 ) and s1 == asId then
				return c_time + 1.500 + db.asPrio
			else
				return GetCooldown("as",asId)						
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2
			if ( s_gc ) and ( s_ha ) then
				c_hp = min(5, c_hp + 3)				-- AS during GC grants 3 HP during HA
			elseif ( s_gc ) then							
				c_hp = min(5, c_hp + 1)				-- AS during GC without HA 
			end
			s_gc = false									-- assumes that players use AS when AS is s1
		end,
		info = "Avenger's Shield",
	},
	asgc = {												
		id = asId,											
		GetCD = function()
			if ( s_gc ) then 
				if ( s1 ) and s1 == asId then
					return c_time + 1.500 + db.asgcPrio
				else
					return GetCooldown("asgc",asId)
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			if ( s_ha ) then
				c_hp = min(5, c_hp + 3)				-- AS during GC grants 3 HP during HA
			else
				c_hp = min(5, c_hp + 1)				-- AS during GC without HA 
			end
			s_gc = false									-- assumes that players use AS when AS is s1
		end,
		info = "Avenger's Shield during Grand Crusader",
	},
	cons = {												-- local-cast ID 26573 
		id = consId,										-- targeted ID 116467 
		GetCD = function()
			if ( s_goc ) and SpellIsTargeting(consId) then 	-- returns nil if spell is -not- waiting for player to select target
				return later 								-- returns -1 if spell is waiting for player to select target 
			elseif ( s1 ) and s1 ~= consId then 
				if ( UnitPower("player",0) / UnitPowerMax("player",0) * 100 ) >= db.consMana then
					return GetCooldown("cons",consId)
				else
					return later 
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
		end,
		info = "Consecration",
	},
	cs = {													
		id = csId,											
		GetCD = function()
			if ( s1 ) and s1 == csId then
				return c_time + 1.500 + db.csPrio
			else
				return GetCooldown("cs",csId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			if ( s_ha ) then 								
				c_hp = min(5, c_hp + 3)				-- CS with HA 
			else
				c_hp = min(5, c_hp + 1)				-- CS without HA 	
			end
		end,
		info = "Crusader Strike",
	}, 
	efc = {								-- efc uses a free cast regardless of HP if conditions to use it are true
		id = efId,											
		GetCD = function()
			local healthPct = ( UnitHealth("player") / UnitHealthMax("player") ) * 100 
			if healthPct <= 20 and (c_hp > 0 or ( s_4t16 ) or ( s_dp )) then 
				return GetCooldown("efc",efId)			
			elseif ( s1 ) and s1 == efId then	
				return c_time + 1.000 + db.efcPrio
			elseif ( not s_ef ) and (c_hp >= db.efcxpHP or ( s_4t16 ) or ( s_dp )) then 
				return GetCooldown("efc.xp",efId)
			-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
			elseif ( s_4t16 ) then	
				if r_ef <= db.efc4T16Refresh and c_bog >= db.efc4T16BoG then 
					return GetCooldown("efc.4t16",efId)
				-- BoG expiration
				elseif ( s_bog ) and c_bog >= db.efc4T16BoGstk and r_bog <= db.efc4T16BoGxp then
					return GetCooldown("efc.4t16",efId)
				else
					return later
				end 
			elseif ( s_dp ) then
				if r_ef <= db.efcdpRefresh and c_bog >= db.efcdpBoG then 
					return GetCooldown("efc.dp",efId)
				-- BoG Expiration 
				elseif ( s_bog ) and c_bog >= db.efcdpBoGstk and r_bog <= db.efcdpBoGxp then
					return GetCooldown("efc.dp",efId)
				else
					return later
				end
			elseif ( s_ha ) then 
				if c_hp >= db.efchaHP and r_ef <= db.efchaRefresh and c_bog >= db.efchaBoG then 
					return GetCooldown("efc.ha",efId)
				-- BoG Expiration 
				elseif ( s_bog ) and c_hp >= db.efchaBoGhp and c_bog >= db.efchaBoGstk and r_bog <= db.efchaBoGxp then
					return GetCooldown("efc.ha",efId)
				else
					return later
				end 
			elseif ( s_swt ) then 							-- WoW 6.0.2 passive talent enables Holy Wrath to produce 1 HP
				if c_hp >= db.efcswHP and r_ef <= db.efcswRefresh and c_bog >= db.efcswBoG then 
					return GetCooldown("efc.sw",efId)
				-- BoG Expiration 
				elseif ( s_bog ) and c_hp >= db.efcswBoGhp and c_bog >= db.efcswBoGstk and r_bog <= db.efcswBoGxp then
					return GetCooldown("efc.sw",efId)
				else
					return later
				end 
			elseif c_hp >= db.efcHP and r_ef <= db.efcRefresh and c_bog >= db.efcBoG then 
					return GetCooldown("efc",efId)
			-- BoG Expiration 
			elseif ( s_bog ) and c_hp >= db.efcBoGhp and c_bog >= db.efcBoGstk and r_bog <= db.efcBoGxp then
				return GetCooldown("efc",efId)
			else
				return later
			end
		end,
		UpdateStatus = function()
			----------------								-- EF is off the GCD
			c_hp = max(0, c_hp - 3)							-- EF consumes all HP available, max of 3
			if ( s_4t16 ) then 
				s_4t16 = false
			elseif ( s_dp ) then
				s_dp = false
			end
			s_bog = false
		    c_bog = 0
		end,		
		info = "Continuous Eternal Flame with configured HP, refresh & BoG",
		reqTalent = 17583,
	},
	efg = {								-- efg uses a free cast regardless of HP if conditions to use it are true	
		id = efId,											
		GetCD = function()
			local c_health	= UnitHealth("player")
			local maxHealth = UnitHealthMax("player")
			local healthPct = ( c_health / maxHealth ) * 100
			-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
			if healthPct <= 20 and (c_hp > 0 or ( s_4t16 ) or ( s_dp )) then 
				return GetCooldown("efg",efId)
			elseif ( s1 ) and s1 == efId then 
				return c_time + 1.000 + db.efgPrio
			elseif ( useEFGalt ) then 
				if ( s_4t16 ) and c_bog >= db.efg4T16AltBoG and ( c_health + c_efHaO ) <= ( db.efg4T16AltHealth / 100 ) * maxHealth then		
					return GetCooldown("efgalt.4t16",efId)
				elseif ( s_dp ) and c_bog >= db.efgdpAltBoG and ( c_health + c_efHaO ) <= ( db.efgdpAltHealth / 100 ) * maxHealth then 
					return GetCooldown("efgalt.dp",efId)
				elseif ( s_ha ) and c_hp >= db.efghaAltHP and c_bog >= db.efghaAltBoG and ( c_health + c_efHaO ) <= ( db.efghaAltHealth / 100 ) * maxHealth then 
					return GetCooldown("efgalt.ha",efId)
				elseif ( s_swt ) and c_hp >= db.efgswAltHP and c_bog >= db.efgswAltBoG and ( c_health + c_efHaO ) <= ( db.efgswAltHhealth / 100 ) * maxHealth then 
					return GetCooldown("efgalt.sw",efId)
				elseif ( not s_dp ) and ( not s_ha ) and ( not s_swt ) and ( not s_4t16 ) and c_hp >= db.efgAltHP and c_bog >= db.efgAltBoG and ( c_health + c_efHaO ) <= ( db.efgAltHealth / 100 ) * maxHealth then 
					return GetCooldown("efgalt",efId)
				else
					return later
				end
			else
				if ( s_4t16 ) and c_bog >= db.efg4T16BoG and healthPct <= db.efg4T16Health then		
					return GetCooldown("efg.4t16",efId)
				elseif ( s_dp ) and c_bog >= db.efgdpBoG and healthPct <= db.efgdpHealth then 
						return GetCooldown("efg.dp",efId)
				elseif ( s_ha ) and c_hp >= db.efghaHP and c_bog >= db.efghaBoG and healthPct <= db.efghaHealth then 
					return GetCooldown("efg.ha",efId)
				elseif ( s_swt ) and c_hp >= db.efgswHP and c_bog >= db.efgswBoG and healthPct <= db.efgswHealth then 
					return GetCooldown("efg.sw",efId)
				elseif ( not s_dp ) and ( not s_ha ) and ( not s_swt ) and ( not s_4t16 ) and c_hp >= db.efgHP and c_bog >= db.efgBoG and healthPct <= db.efgHealth then 
					return GetCooldown("efg",efId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			----------------								-- EF is off the GCD
	  		c_hp = max(0, c_hp - 3)							-- EF consumes all HP available, max of 3
			if ( s_4t16 ) then 
				s_4t16 = false
			elseif ( s_dp ) then
				s_dp = false
			end
			s_bog = false
		    c_bog = 0
		end,		
		info = "Glorious Eternal Flame with configured HP, BoG & Health",
		reqTalent = 17583,
	},    
	es = {
		id = esId,
		GetCD = function()
			if ( s1 ) and s1 ~= esId then 
				return GetCooldown("es",esId)
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2	
		end,
		info = "Execution Sentence",
		reqTalent = 17609,
	},
	hotr = {													
		id = hotrId,											
		GetCD = function()
			if ( s1 ) and s1 == hotrId then						
				return c_time + 1.500 + db.hotrPrio
			else
				return GetCooldown("hotr",hotrId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2
			if ( s_ha ) then 
				c_hp = min(5, c_hp + 3)				-- HotR with HA
			else
				c_hp = min(5, c_hp + 1)				-- HotR without HA
			end
		end,
		info = "Hammer of the Righteous",
	},
	how = {	
		id = howId,
		GetCD = function()
			if IsUsableSpell(howId) then 
				if ( s1 ) and s1 == howId then
					return c_time + 1.500 + db.howPrio
				else
					return GetCooldown("how",howId)
				end
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2
		end,
		info = "Hammer of Wrath",
	},
	hpr = {
		id = hprId,
		GetCD = function()
			if ( s1 ) and s1 ~= hprId then 
				return GetCooldown("hpr",hprId)
			else
				return later
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
		end,
		info = "Holy Prism",
		reqTalent = 17605,
	},
	hw = {
		id = hwId,
		GetCD = function()
			if ( s1 ) and s1 ~= hwId then
				return GetCooldown("hw",hwId)
			else
				return later								-- hwsw, hwf, or hw already chosen for s1
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			if ( s_swt ) then c_hp = c_hp + 1 end
		end,
		info = "Holy Wrath",
	},
	hwf = {													-- action added 
		id = hwId,
		GetCD = function()
			if ( s1 ) and s1 == hwId then					-- hwsw, hwf, or hw already chosen for s1
				return c_time + 1.500 + db.hwfPrio
			else 
				local targetHealthPercent = GetTargetHealthPercent()	-- returns zero if there is no target
				if targetHealthPercent > 0 and targetHealthPercent <= 20 then 
					return GetCooldown("hwf",hwId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			if ( s_swt ) then c_hp = c_hp + 1 end
		end,
		info = "Holy Wrath with Glyph of Final Wrath",
		reqGlyph = 54935,
	},
	hwsw = {
		id = hwId,
		GetCD = function()
			if ( s1 ) and s1 == hwId then					-- hwsw, hwf, or hw already chosen for s1
				return c_time + 1.500 + db.hwswPrio
			else
				return GetCooldown("hwsw",hwId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			c_hp = c_hp + 1 
		end,
		info = "Holy Wrath with Sanctified Wrath",
		reqTalent = 21795,
	},
	j = {													
		id = jId,											
		GetCD = function()
			if ( s1 ) and s1 == jId then 
				return c_time + 1.500 + db.jPrio
			else 
				return GetCooldown("j",jId)
			end
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2 
			if ( s_ha ) then				
				c_hp = min(5, c_hp + 3)				-- J grants 3 HP during HA, but forgoes 1 or 2 when HP > 2
			else
				c_hp = min(5, c_hp + 1)				-- J without HA 
			end
		end,
		info = "Judgment",
	},
	lh = {	
		id = lhId,
		GetCD = function()
			if SpellIsTargeting(lhId) then				-- returns nil if spell is not waiting for player to select target
				return later							-- returns -1 if spell is waiting for player to select target 
			elseif ( s1 ) and s1 ~= lhId then
				return GetCooldown("lh",lhId)
			else
				return later
			end 
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2
		end,
		info = "Light's Hammer",
		reqTalent = 17607,
	},
	sotr = {													-- so configurable it makes the mind spin
		id = sotrId,
		GetCD = function()
 			if ( s1 ) and s1 == sotrId then 
 				return c_time + 1.000 + db.sotrPrio
			elseif ( s_efc ) and ( not s_ef ) and ( c_hp <= db.efcxpHP or ( s_dp )) and ( not s_4t16 ) then
				return later
			-- use SotR to consume DP free cast when a 4-pc T16 free cast is also available 
			elseif ( s_4t16 ) then 
				if c_hp >= db.sotr4t16HP or ( s_dp ) then 
					return GetCooldown("sotr.4t16 ",sotrId)
				-- maintain the BoG stack - see also efc during HA
				elseif ( s_bog ) and c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr.4t16bog",sotrId)
				else
					return later
				end
			elseif ( s_dp ) then 
				if c_hp >= db.sotrdpHP then	
					return GetCooldown("sotr.dp",sotrId)
				-- maintain the BoG stack - see also efc during DivPurp
				elseif ( s_bog ) and c_bog >= db.sotrBoGstk and r_bog <= db.sotrBoGxp then 
					return GetCooldown("sotr.dpBoG",sotrId)
				elseif r_dp <= db.sotr_dptr then				-- perchance the free cast has yet to be used 
					return GetCooldown("sotr.dpExp",sotrId)
				else
					return later 
				end 
			elseif ( s_ha ) then 
				if c_hp >= db.sotrhaHP then				-- settings range 3 - 5 HP
					return GetCooldown("sotr.ha",sotrId)
				-- maintain the BoG stack - see also efc during HA
				elseif ( s_bog ) and c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr.habog",sotrId)
				else
					return later
				end
			elseif ( s_swt ) then 
				if c_hp >= db.sotrswHP then		-- settings range 3 - 5 HP
					return GetCooldown("sotr.sw",sotrId)
				-- maintain the BoG stack - see also efc during SW
				elseif ( s_bog ) and c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr.swbog",sotrId)
				else
					return later
				end
			elseif c_hp >= db.sotrHP then 					-- the Ordinary case (range 3 - 5) :-)
				return GetCooldown("sotr",sotrId)
			-- maintain the BoG stack -- see also efc ordinary case
			elseif ( s_bog ) and c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog <= db.sotrBoGxp then
				return GetCooldown("sotr.bog",sotrId)
			else
				return later
			end
		end, 
		UpdateStatus = function()
			----------------								-- SotR is off the GCD
			if ( s_dp ) then
				s_dp = false 								-- assumes that player uses SotR when SotR is s1
			else 
				c_hp = max(0, c_hp - 3) 					-- the Ordinary case ... currently sotr requires 3 HP to cast ...
			end
			s_bog = true
			c_bog = c_bog + 1 
		end, 
		info = "Shield of the Righteous (configured)",  
	},
	sotrdp = {	
		id = sotrId,
		GetCD = function()
			if ( s_dp ) then 
				if ( s1 ) and s1 == sotrId then 
					return c_time + 1.000 + db.sotrxxPrio
				elseif ( s_efc ) and ( not s_ef ) and ( c_hp >= db.efcxpHP or ( s_4t16 )) then
					return GetCooldown("sotr.dp",sotrId)					
				elseif c_hp >= db.sotrdpHP or ( s_4t16 ) then	
					return GetCooldown("sotr.dp",sotrId)
				-- increase or maintain the BoG stack - see also efc during DivPurp
				elseif c_bog >= db.sotrBoGstk and r_bog >= db.r_bog_min and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr.dp",sotrId)
				elseif r_dp <= db.sotr_dptr then				-- perchance the free cast has yet to be used 
					return GetCooldown("sotr.dp",sotrId)
				else
					return later 
				end 
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								-- SotR is off the GCD
			s_dp = false 									-- assumes that player uses SotR when chosen for s1
			s_bog = true
			c_bog = c_bog + 1 
		end,
		info = "Priortize Shield of the Righteous during Divine Purpose",
	},
	sotrha = {												-- sorha recommended to use sotr with HA 										
		id = sotrId,
		GetCD = function()
			if ( s_ha ) then 
				if ( s1 ) and s1 == sotrId then
					return c_time + 1.000 + db.sotrxxPrio
				elseif ( s_efc ) and ( not s_ef ) and db.efcRefresh > 0 and c_hp <= db.efcHP then
					return later
				elseif c_hp >= db.sotrhaHP then 
					return GetCooldown("sotrha",sotrId)
				-- increase or maintain the BoG stack - see also efc during HA
				elseif c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog >= db.r_bog_min and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr",sotrId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								-- SotR is off the GCD
			c_hp = max(0, c_hp - 3)
			s_bog = true
			c_bog = c_bog + 1
		end,
		info = "Priortize Shield of the Righteous during Holy Avenger",
	},
	sotrsw = {												
		id = sotrId,
		GetCD = function() 
			if ( s_swt ) then 
				if ( s1 ) and s1 == sotrId then 
					return c_time + 1.000 + db.sotrxxPrio
				elseif ( s_efc ) and ( not s_ef ) and db.efcRefresh > 0 and c_hp <= db.efcHP then
					return later
				elseif c_hp >= db.sotrswHP then
					return GetCooldown("sotrsw",sotrId)
				-- increase or maintain the BoG stack - see also efc during SW
				elseif c_hp >= 3 and c_bog >= db.sotrBoGstk and r_bog >= db.r_bog_min and r_bog <= db.sotrBoGxp then
					return GetCooldown("sotr",sotrId)
				else
					return later
				end
			else
				return muchlater
			end
		end,
		UpdateStatus = function()
			----------------								-- SotR is off the GCD
			c_hp = max(0, c_hp - 3)
			s_bog = true
			c_bog = c_bog + 1
		end,
		info = "Priortize Shield of the Righteous with Sanctified Wrath",
	},
	ss = {
		id = sasId,
		GetCD = function()
			if ( s1 ) and s1 == sasId then 
				if db.ssPrio then 							-- ToDo: is priority time causing problems?
					return c_time + 1.000 + db.ssPrio 
				else 
					return later 
				end
			elseif ( s_ss ) and r_ss <= db.ssRefresh then
				return GetCooldown("ss",sasId)
			elseif ( not s_ss ) then
				return GetCooldown("ss",sasId)
			else
				return later
			end 
		end,
		UpdateStatus = function()
			c_time = c_time + gcds2
			r_ss = muchlater -- fix to stop sacred shield from showing up immediately after use (--dA: prior source r_ss = 100)
		end,
		info = "Sacred Shield",
		reqTalent = 21811,
	},	
	wog = {								-- wog uses DP free cast regardless of HP, if its other conditions are true
		id = wogId,											
		GetCD = function()
			local c_health	= UnitHealth("player")
			local maxHealth = UnitHealthMax("player")
			local healthPct = c_health / maxHealth * 100 
			if healthPct <= 20 and (c_hp > 0 or ( s_dp ) or ( s_4t16)) then 
				return GetCooldown("wog",wogId)
			elseif ( s1 ) and s1 == wogId then
				return c_time + 1.000 + db.wogPrio
			elseif ( useWoGalt ) then
			-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
				if ( s_4t16 ) and c_bog >= db.wog4T16AltBoG and ( c_health + c_wogHaO ) <= ( db.wog4T16AltPct / 100 ) * maxHealth then 
					return GetCooldown("wogalt.4t16",efId)
				elseif ( s_dp ) and c_bog >= db.wogdpAltBoG and ( c_health + c_wogHaO ) <= ( db.wogdpAltPct / 100 ) * maxHealth then 
					return GetCooldown("wogalt.dp",efId)
				elseif ( s_ha ) and c_hp >= db.woghaAltHP and c_bog >= db.woghaAltBoG and ( c_health + c_wogHaO ) <= ( db.woghaAltPct / 100 ) * maxHealth then 
					return GetCooldown("wogalt.ha",efId)
				elseif ( s_swt ) and c_hp >= db.wogswAltHP and c_bog >= db.wogswAltBoG and ( c_health + c_wogHaO ) <= ( db.wogswAltPct / 100 ) * maxHealth then 
					return GetCooldown("wogalt.sw",efId)
				elseif ( not s_dp ) and ( not s_ha ) and ( not s_swt ) and c_hp >= db.wogAltHP and c_bog >= db.wogAltBoG and ( c_health + c_wogHaO ) <= (( db.wogAltPct / 100 ) * maxHealth ) then 
					return GetCooldown("wogalt",efId)
				else
					return later
				end
			else
			-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
				if ( s_4t16 ) and c_bog >= db.wog4T16BasStack and healthPct <= db.wog4T16Percent then 
					return GetCooldown("wog.4t16",efId)
				elseif ( s_dp ) and c_bog >= db.wogdpBasStack and healthPct <= db.wogdpPercent then
					return GetCooldown("wog.dp",wogId)
				elseif ( s_ha ) and c_hp >= db.woghaHp and c_bog >= db.woghaBasStack and healthPct <= db.woghaPercent then 
					return GetCooldown("wog.ha",wogId)
				elseif ( s_swt ) and c_hp >= db.wogswHp and c_bog >= db.wogswBasStack and healthPct <= db.wogswPercent then 
					return GetCooldown("wog.sw",wogId)
				elseif ( not s_dp) and ( not s_ha ) and ( not s_w ) and c_hp >= db.wogHp and c_bog >= db.wogBasStack and healthPct <= db.wogPercent then
					return GetCooldown("wog",wogId)
				else
					return later
				end
			end
		end,
		UpdateStatus = function()
			----------------								-- WoG is off the GCD
			if ( s_dp ) then 								-- WoG does not consume HP during DP
				s_dp = false								-- assumes that player uses WoG when wog is s1
			else
				c_hp = max(0, c_hp - 3)						-- WoG consumes all available HP, max of 3
			end 
			s_bog = false
			c_bog = 0
		end,
		info = "Word of Glory with configured HP, BoG and Health",
	},
	wogdp = {
		id = wogId,
		GetCD = function()
			if ( not s_dp ) then
				return later
			else
				local c_health	= UnitHealth("player")
				local maxHealth = UnitHealthMax("player")
				local healthPct = c_health / maxHealth * 100 
				if healthPct <= 20 then 					-- free cast is available for this action
					return GetCooldown("wogdp.20",wogId)
				elseif ( s1 ) and s1 == wogId then
					return c_time + 1.000 + db.wogxxPrio
				elseif ( useWoGalt ) then 
				-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
					if ( s_4t16 ) and c_bog >= db.wog4T16AltBoG and ( c_health + c_wogHaO ) <= ( db.wog4T16AltPct / 100 ) * maxHealth then 
						return GetCooldown("wogdpalt.4t16",efId)
					elseif c_bog >= db.wogdpAltBoG and ( c_health + c_wogHaO ) <= ( db.wogdpAltPct / 100 ) * maxHealth then 
						return GetCooldown("wogdpalt",efId)
					else
						return later
					end
				else
				-- if both the 4-pc Tier 16 set free cast and a DP free cast are available, the 4-pc Tier 16 set free cast is used
					if ( s_4t16 ) and c_bog >= db.wog4T16BasStack and healthPct <= db.wog4T16Percent then 
						return GetCooldown("wogdp.4t16",efId)
					elseif c_bog >= db.wogdpBasStack and healthPct <= db.wogdpPercent then
						return GetCooldown("wogdp",wogId)
					else
						return later
					end
				end 
			end
		end,
		UpdateStatus = function()
			----------------								-- WoG is off the GCD
			----------------								-- WoG consumes 0 HP during DP
			s_dp = false									-- assumes that player uses WoG when wogdp is s1
			s_bog = false
			c_bog = 0
		end,
		info = "Priortize Word of Glory during Divine Purpose",
	},
	wogha = {	
		id = wogId,
		GetCD = function()
			if ( s_ha ) then 
				local c_health	= UnitHealth("player")
				local maxHealth = UnitHealthMax("player")
				local healthPct = c_health / maxHealth * 100 
				if healthPct <= 20 and ( c_hp > 0 or (s_4t16) ) then 		-- DP proc is unavailable while HA is the L75 talent
					return GetCooldown("wogha.20",efId)
				elseif ( s1 ) and s1 == wogId then 
					return c_time + 1.000 + db.wogxxPrio
				elseif ( useWoGalt ) then 
					if ( s_4t16 ) and c_bog >= db.wog4T16AltBoG and ( c_health + c_wogHaO ) <= ( db.wog4T16AltPct / 100 ) * maxHealth then 
						return GetCooldown("woghaalt.4t16",efId)
					elseif c_hp >= db.woghaAltHP and c_bog >= db.woghaAltBoG and ( c_health + c_wogHaO ) <= ( db.woghaAltPct / 100 ) * maxHealth then 
						return GetCooldown("woghaalt",efId)
					else
						return later
					end
				else
					if ( s_4t16 ) and c_bog >= db.wog4T16BasStack and healthPct <= db.wog4T16Percent then 
						return GetCooldown("wogha.4t16",efId)
					elseif c_hp >= db.woghaHp and c_bog >= db.woghaBasStack and healthPct <= db.woghaPercent then
						return GetCooldown("wogha",wogId)
					else
						return later
					end 
				end
			else
				return later
			end 
		end,
		UpdateStatus = function()
			----------------								-- WoG is off the GCD
			c_hp = max(0, c_hp - 3)							-- WoG consumes all available HP, max of 3
			s_bog = false
			c_bog = 0
		end,
		info = "Priortize Word of Glory during Holy Avenger",
	},
	wogsw = {
		id = wogId,
		GetCD = function()
            if ( s_swt ) then 
				local healthPct = UnitHealth("player") / UnitHealthMax("player") * 100 
				if healthPct <= 20 and c_hp >= 1 then 
					return GetCooldown("wogsw.20",efId)
				elseif ( s1 ) and s1 == wogId then 
					return c_time + 1.000 + db.wogxxPrio
				elseif ( useWoGalt ) then 
					if ( s_4t16 ) and c_bog >= db.wog4T16AltBoG and ( c_health + c_wogHaO ) <= ( db.wog4T16AltPct / 100 ) * maxHealth then 
						return GetCooldown("wogswalt.4t16",efId)
					elseif c_hp >= db.wogswAltHP and c_bog >= db.wogswAltBoG and ( c_health + c_wogHaO ) <= ( db.wogswAltPct / 100 ) * maxHealth then 
						return GetCooldown("wogswalt",efId)
					else
						return later
					end 
				else
					if ( s_4t16 ) and c_bog >= db.wog4T16BasStack and healthPct <= db.wog4T16Percent then 
						return GetCooldown("wogsw.4t16",efId)
					elseif c_hp >= db.wogswHp and c_bog >= db.wogswBasStack and healthPct <= db.wogswPercent then
						return GetCooldown("wogsw",wogId)
					else
						return later
					end
				end
			else
				return later
			end 
		end,
		UpdateStatus = function()
			----------------								-- WoG is off the GCD
			c_hp = max(0, c_hp - 3)							-- WoG consumes all available HP, max of 3
			s_bog = false
			c_bog = 0
		end,
		info = "Priortize Word of Glory with Sanctified Wrath",
	},
}

clcprot.RR_actions = actions 

-----------------------------------------------------------------------------------------------------------------------
-- action priority sequence & rotation functions 
-----------------------------------------------------------------------------------------------------------------------

function clcprot.RR_UpdateQueue()							-- function called in clcprot.lua during Init 

	-- KEEP THIS FUNCTION IN THIS FILE - USING IT IN CLCPROT.LUA CAUSES !SWATTER & BLIZZ TO (FALSELY) REPORT AN "ERROR" 
	-- set the current db
	local db = clcprot.db.profile.rotation

	-- normal queue
	if db.prio == "" then db.prio = db.prio_def end
	qn = {}
	for v in string.gmatch(db.prio, "[^ ]+") do
		if actions[v] then
			table.insert(qn, v)
		else
			print("CLCProt: invalid action in normal priority sequence: ", v)
		end
	end
	db.prio = table.concat(qn, " ")

	-- alternate queue
	if db.prioAlt == "" then db.prioAlt = db.prioAlt_def end
	qalt = {}
	for v in string.gmatch(db.prioAlt, "[^ ]+") do
		if actions[v] then
			table.insert(qalt, v)
		else
			print("CLCProt: invalid action in alternate priority sequence: ", v)
		end
	end
	db.prioAlt = table.concat(qalt, " ")

	-- flag to signal GetNextAction() to call GetWorkingQue() after queue is loaded or changed
	s_queue = true

end

local function GetWorkingQueue()

	local name, selected, available
	s_efc, s_efg, s_wog = false, false, false			-- flags for actions used in decisions 

	if useAltPrio then									-- useAltPrio is a global variable in clcprot.lua
		-- alternate queue
		q = {}
		for k, v in pairs(qalt) do
			-- check for talent requirement
			if actions[v].reqTalent then
				-- check whether talent is active
				_, name, _, selected, available = GetTalentInfoByID(actions[v].reqTalent, talentGroup)
				if name and selected and available then		--dA: abija's tests
					if v == "efc" then 
						s_efc = true
						table.insert(q, v)
					elseif v == "efg" then
						s_efg = true
						table.insert(q, v)
					else 
						table.insert(q, v)
					end
				end
			-- check for glyph requirement					--dA: not used for Glyph of Consecration
			elseif actions[v].reqGlyph then
				-- check whether glyph is socketed 
				if IsGlyphInSocket(actions[v].reqGlyph) then		--dA: in clcprot.lua, returns true/false
					table.insert(q, v)
				end
			elseif ( s_eft ) then 
				local flag = false
				flag = string.match("ss wog wogdp wogha wogsw", v)
				if ( not flag ) then table.insert(q, v); end 
			else	------------------------------------------ EF is not selected as the Tier 3 Level 45 talent 
				if v == "wog" or v == "wogha" or v == "wogsw" or v == "wogdp" then 
					s_wog = true
					table.insert(q, v)
				else
					local flag = false
					flag = string.match("efc efg", v)
					if ( not flag ) then table.insert(q, v); end 
				end
			end 
		end
		if ( s_initPrio) or ( prioToggled ) then 
			print("CLCProt: priority sequence is Alternate: ",table.concat(q," "))
			s_initPrio = false
			prioToggled = "no"
		end
	else	
		-- normal queue
		q = {}
		for k, v in pairs(qn) do
			-- check for talent requirement
			if actions[v].reqTalent then
				-- check whether talent is active
				_, name, _, selected, available = GetTalentInfoByID(actions[v].reqTalent, talentGroup)
				if name and selected and available then		--dA: abija's tests
					if v == "efc" then 
						s_efc = true
						table.insert(q, v)
					elseif v == "efg" then
						s_efg = true
						table.insert(q, v)
					else 
						table.insert(q, v)
					end
				end
			-- check for glyph requirement					--dA: not used for Glyph of Consecration
			elseif actions[v].reqGlyph then
				-- check whether glyph is socketed 
				if IsGlyphInSocket(actions[v].reqGlyph) then		--dA: in clcprot.lua, returns true/false
					table.insert(q, v)
				end
			elseif ( s_eft ) then 
				local flag = false
				flag = string.match("ss wog wogdp wogha wogsw", v)
				if ( not flag ) then table.insert(q, v); end 
			else	-- EF is not selected as the Tier 3 Level 45 talent 
				if v == "wog" or v == "wogha" or v == "wogsw" or v == "wogdp" then 
					s_wog = true
					table.insert(q, v)
				else
					local flag = false
					flag = string.match("efc efg", v)
					if ( not flag ) then table.insert(q, v); end 
				end
			end 
		end
		if ( s_initPrio) or ( prioToggled ) then 
			print("CLCProt: priority sequence is Normal: ",table.concat(q," "))
			s_initPrio = false
			prioToggled = "no"
		end
	end 
	return
end

local function GetEternalFlameHealing(hpAvail)						-- new function 

	-- returns the current amounts of the EF heal-at-once and the EF heal-over-time that are available
	-- BoG healing bonus per unit is 6% plus 75% of the paladin's current Mastery percentage 
	-- BoG applies only to HaO, and only when EF is self-cast (self-cast is assumed)
	-- HoT bonus is increased 50% when EF is cast on the paladin (self-cast is assumed)

	local efHaO, efHoT, c_mastery, bogBuff, palaHoT, efHaOamt, efHoTamt = 0, 0, 0, 0, 0, 0, 0

	local base, posBuff, negBuff = UnitAttackPower("player")
	if hpAvail and base and posBuff and negBuff then 
		local SpellPower = ( base + posBuff + negBuff )
		efHaO = 2.64591 * SpellPower
		efHoT = 0.10 * SpellPower 	
		c_mastery = GetMastery() / 100			-- mastery returned as whole percentage xx.xxxxxxx
		if c_mastery then 
			bogBuff = (0.06 + (0.75 * c_mastery)) * c_bog
			efHaOamt = (efHaO + (bogBuff * efHaO)) * hpAvail
			efHoTamt =  efHoT * 1.5				-- HoT increased 50% when cast on the paladin 
		else
			print("CLCProt: rotation [",rota,"] GetEternalFlameHealing(): invalid c_mastery: ",c_mastery)
		end
	else
		print("CLCProt: rotation [",rota,"] GetEternalFlameHealing(): invalid quantity hpAvail: ",hpAvail," base: ",base," posBuff: ",posBuff," negBuff: ",negBuff)
	end
--	if rota == 1 or rota % 15 == 0 then 
--	print("CLCProt: rotation [",rota,"] GetEternalFlameHealing(): hpAvail: ",hpAvail," c_mastery: ",c_mastery," c_bog: ",c_bog," efHaO: ",efHaO," efHaOamt: ",efHaOamt," efHoT: ",efHoT," efHoTamt: ",efHoTamt)
--	end
	return efHaOamt, efHoTamt
end

local function GetWordOfGloryHealing(hpAvail)				-- new function 

	-- returns the current amount of WoG healing available
	-- BoG healing bonus per unit is 6% plus 75% of the paladin's current Mastery percentage
	-- BoG bonus applies only if the WoG is cast on the paladin (self-cast is assumed) 

	local wogHaO, c_mastery, bogBuff, wogHaOamt = 0, 0, 0 

	local base, posBuff, negBuff = UnitAttackPower("player")
	if hpAvail and base and posBuff and negBuff then 
		local SpellPower = ( base + posBuff + negBuff )
		wogHaO = 2.64591 * SpellPower 
		c_mastery = GetMastery() / 100						-- mastery returned as a whole percentage xx.xxxxxxx
		if c_mastery then 
			bogBuff = (0.06 + (0.75 * c_mastery)) * c_bog
			wogHaOamt = (wogHaO + (bogBuff * wogHaO)) * hpAvail
		else
			print("CLCProt: rotation [",rota,"] GetWordOfGloryHealing(): invalid c_mastery: ",c_mastery)
		end
	else
		print("CLCProt: rotation [",rota,"] GetWordOfGloryHealing(): invalid quantity hpAvail: ",hpAvail," base: ",base," posBuff: ",posBuff," negBuff: ",negBuff)
	end
--	if rota == 1 or rota % 15 == 0 then 
--	print("CLCProt: rotation [",rota,"] GetWordOfGloryHealing(): hpAvail: ",hpAvail," c_mastery: ",c_mastery," c_bog: ",c_bog," c_wogHaO: ",wogHaO," wogHaOamt: ",wogHaOamt)
--	end
	return wogHaOamt
end

--[[
local function GetDebuff(buffname)							-- formerly used for Weakened Blows debuff to tank target

	-- GetBuff() returns (a) true if the buff exists, (b) stack count, and (c) the time that buff expires

	local nflag = false
	
	if buffname and type(buffname) == "string" and buffname ~= "" then 
	
		local d_name, _, _, d_count, _, _, d_expires = UnitDebuff("target", buffname, nil, "TARGET")

		if d_name then	
			nflag = true
		else												-- debuff not found on target (d_name is nil)
			d_count = 0.000									-- don't return nil (!)
			d_expires = 0.000								-- don't return nil (!)
		end
		return nflag, d_count, d_expires					-- time left in seconds to millisecond precision
	else
		print("CLCProt rotation [",rota,"]: buff name is invalid for GetDebuff(",buffname,").")		
		return nflag, 0, 0.000
	end
end
--]]

local function GetBuff(buffname)							-- prior source replaced

	-- GetBuff() returns (a) true if the buff exists, (b) stack count, and (c) the time that buff expires

	-- While Sanctified Wrath is active, the WoW client displays an icon for it among the buffs on the default UI. However,
	----: UnitBuff() returns zero for the Sanctified Wrath expiration time. Since it does return the name, whether Sanctified 
	----: Wrath is in effect can be flagged. 
	----: reference copy: local name, _, _, stack, _, _, expires, _, _, _, spellId = UnitBuff("player", buffname)

	local nflag = false

	if buffname and type(buffname) == "string" and buffname ~= "" then 

		local name, _, _, stack, _, _, expires = UnitBuff("player", buffname)

		if name then 
			nflag = true								-- buff exists on the paladin
			return nflag, stack, expires				-- if buff does not stack, then stack = 0 (documented for UnitDebuff)
		else
			return nflag, 0, 0.000						-- buff does not exist on the unit, nflag is false (default)
		end
	else
		print("CLCProt rotation [",rota,"]: buff name is invalid for GetBuff(",buffname,").")
		return nflag, 0, 0.000
	end
end

local function GetGlobalCooldown()	

	-- This function returns (1) the duration, and (2) the time that the GCD will expire
	----: if a GCD is not in progress, the duration is zero, and the time of expiration is zero
	----: if the duration is more than zero but less than the Paladin minimum, that minimum is retured
	----:
	-- GCD = 1.5 / ( (1 + (haste% / 100)) * other haste buffs ...)  	[to quote Fouton]
	----:
	---------------------------------------------------------------------------=

	count_getGCD = count_getGCD + 1							-- debugging
	local start, duration = 0.000, 0.000

	if id and type(id) == "number" and id > 0 then			-- test whether id is nil and positive number 
		
		start, duration = GetSpellCooldown(id)				-- returns start = 0 and duration = 0 if GCD is not in progress
--		local xp_gcd = start + duration
--		print("GCD(",id,") st: ",start," dur: ",duration," exp: ",xp_gcd)

		if start and duration then			 				-- GetSpellCooldown(id) returns nils if nonesuch spell id
			if duration > 0 and duration < 1.000 then duration = 1.000 end	-- adjust for paladin minimum GCD
			return duration, start + duration
		else
			print("CLCProt rotation [", rota, "] GetGlobalCooldown(id) nonesuch id on call: ",count_getGCD," id:",id," start: ",start," duration: ",duration)
			return 0,0
		end
	end
	print("CLCProt rotation [",rota,"] GetGlobalCooldown(id) invalid id on call: ",count_getGCD," id:",id," start: ",start," duration: ",duration)
	return 0,0
end

local function GetUpdates()									-- new function to update flags, buffs & debuffs for s2

	-- This function primarily updates the time remaining on each buff, then
	-- compares it to the minimum time to act, and evaluates flags accordingly.
	-- It also determines whether an update to the available healing is needed, 
    -- and obtains the update if necessary.

	-- set the current db
	db = clcprot.db.profile.rotation
	
	-- time for updating expirations
	local t_now = GetTime()

	-- global cooldown
	if t_now >= gcdExp then									-- gcdExp may be zero (returned by previous call)
--		gcdDur, gcdExp = GetGlobalCooldown(gcdId)			-- returns duration & expiration time of current GCD in progress
		gcdExp = t_now + gcdDur								-- gcdDur calculated in GetStatus()
	end

    -- expiration times were previously obtained from GetBuff() and from GetDebuff() via local function GetStatus()
	-- if the buff expires during the current GCD then it will not be in effect with regard to using the next skill
	local s_dp_exp = false									-- DP free cast was used or expired
	local s_4t16_exp = false								-- Tier 16 4-piece free cast was used or BoG expired
	local r_gc, r_ha, r_ser
	if ( s_bog ) then
		r_bog = max(0, x_bog - t_now - gcdDur)				-- time remaining for Bastion of Glory stack
		if r_bog < 0.350 then r_bog = 0; s_bog = false; c_bog = 0; s_bop = false; s_4t16 = false; s_4t16_exp = true end	
	end
	if ( s_dp ) then
		r_dp = max(0, x_dp - t_now - gcdDur)				-- time remaining for Divine Purpose proc
		if r_dp < 0.350 then r_dp = 0; s_dp = false; s_dp_exp = true; end
	end
	if ( s_ef ) then
		r_ef = max(0, x_ef - t_now - gcdDur)				-- time remaining for Eternal Flame buff 
		if r_ef < 0.350 then r_ef = 0; s_ef = false; end	
	end
	if ( s_gc ) then
		r_gc = max(0, x_gc - t_now - gcdDur)				-- time remaining for Grand Crusader proc
		if r_gc < 0.350 then s_gc = false; end
	end
	if ( s_ha ) then 
		r_ha = max(0, x_ha - t_now - gcdDur) 				-- time remaining for Holy Avenger
		if r_ha < 0.350 then s_ha = false; end
	end
	if ( s_ser ) then 
		r_ser = max(0, x_ser - c_time - gcdDur) 			-- time remaining for Seraphim
		if r_ser < 0.350 then s_ser = false; end
	end
	if ( s_ss ) then 
		r_ss = max(0, x_ss - t_now - gcdDur)				-- time remaining for Sacred Shield buff 
		if r_ss < 0.350 then r_ss = 0; s_ss = false; end 
	end

	-- update available healing for actions efg and wog if previous free casts have been used or have expired
	local hpForUse = 0 
	if (( s_dp_exp ) and ( not s_4t16 )) or (( s_4t16_exp ) and (not s_dp)) then 
		hpForUse = min(3, c_hp)
		if ( s_efg ) and hpForUse > 0 then 
			c_efHaO, c_efHoT = GetEternalFlameHealing(hpForUse) 
		elseif ( s_wog ) and  hpForUse > 0 then
			c_wogHaO = GetWordOfGloryHealing(hpForUse)
		end
	end

	return
end

-- READS ALL THE -INTERESTING- DATA !!!

local function GetStatus()									-- revised original source	

	-- set the current db
	db = clcprot.db.profile.rotation

	c_hp = UnitPower("player", SPELL_POWER_HOLY_POWER)		-- prior source
	c_mHaste = GetMeleeHaste() / 100						-- evaluated for calculations

	-- the buffs and debuffs
	-- GetBuff() returns (1) true if the buff exists (s_??); (2) the number of times the buff has been applied; and 
	----: (3) the time that the buff will expire (x_??) in seconds to millisecond precision, a floating-point number, 
	----: with the same frame of reference as GetTime().
	----: 
	-- GetDebuff() returns (1) true if the debuff exists (s_??); (2) the number of times the debuff has been applied, and 
	----: (3) the time that the debuff will expire (x_??) in seconds to millisecond precision, a floating-point number, 
	----: with the same frame of reference as GetTime().
	
	s_bog, c_bog, x_bog = GetBuff(buffBG)	-- BoG stack 
	 s_bop,  _,    _    = GetBuff(buffBP)	-- bastion of power flag is true when the 4-piece Tier 16 set free cast is available
	      s_dp, _, x_dp = GetBuff(buffDP)	-- divine purpose
		  s_ef, _, x_ef = GetBuff(buffEF)	-- eternal flame
		  s_gc, _, x_gc = GetBuff(buffGC)	-- grand crusader	
		  s_ha, _, x_ha = GetBuff(buffHA)	-- holy avenger
		s_ser, _, x_ser = GetBuff(buffSR)	-- seraphim
		  s_ss, _, x_ss = GetBuff(buffSS)	-- sacred shield
--[[
	-- global cooldown
	gcdDur, gcdExp = GetGlobalCooldown(gcdId)				-- returns duration & expiration time of current GCD in progress
	if gcdDur > 0 then gcds2 = gcdDur; 						-- GCD increment to current time for UpdateStatus()
				  else gcds2 = 1.500; end
--]]
	c_time = GetTime()
	later = c_time + howlate 								-- "expiration time" to return if GetCooldown() is not called
	muchlater = later + howlate								-- increase for inactive actions and for error conditions

	-- global cooldown --------------------------------------- while GlobalCooldown spell is broken
	if c_time >= gcdExp then 
		gcdDur = 1.5 / ( 1 + c_mHaste ) 					-- c_mHaste is a decimal percentage (0.xxxx)
		gcdExp = c_time + gcdDur
	end
	gcds2 = gcdDur; 										-- GCD increment to current time for UpdateStatus()
	--------------------------------------------------------

	-- if a buff expires during the current GCD then it will not be in effect with regard to using the next skill afterward
	-- the minimum time to act is the constant 0.350 sec. 

	local r_gc, r_ha, r_ser
	if ( s_bog ) then
		r_bog = max(0, x_bog - c_time - gcdDur)				-- time remaining for Bastion of Glory stack
		if r_bog < 0.350 then s_bog = false; c_bog = 0; s_bop = false; end	
	end
	if ( s_dp ) then
		r_dp = max(0, x_dp - c_time - gcdDur)				-- time remaining for Divine Purpose proc
		if r_dp < 0.350 then r_dp = 0; s_dp = false; end
	end
	if ( s_ef ) then
		r_ef = max(0, x_ef - c_time - gcdDur)				-- time remaining for Eternal Flame buff 
		if r_ef < 0.350 then r_ef = 0; s_ef = false; end	
	end
	if ( s_gc ) then
		r_gc = max(0, x_gc - c_time - gcdDur)				-- time remaining for Grand Crusader proc
		if r_gc < 0.350 then s_gc = false; end
	end
	if ( s_ha ) then 
		r_ha = max(0, x_ha - c_time - gcdDur) 				-- time remaining for Holy Avenger
		if r_ha < 0.350 then s_ha = false; end
	end
	if ( s_ser ) then 
		r_ser = max(0, x_ser - c_time - gcdDur) 			-- time remaining for Seraphim
		if r_ser < 0.350 then s_ser = false; end
	end
	if ( s_ss ) then 
		r_ss = max(0, x_ss - c_time - gcdDur)				-- time remaining for Sacred Shield buff 
		if r_ss < 0.350 then r_ss = 0; s_ss = false; end 
	end
	
	-- check whether armor Tier 16 4-piece set free cast is available
	if ( s_bop ) and c_bog >= 3 then 
		s_4t16 = true 
	else 
		s_4t16 = false
	end

	-- dA: get available healing 
	-- dA: EF or WoG may be managed as a raid cooldown, so action priority sequence is checked for actions efc, efg, wog
---[[	
	local hpForUse = min(3, c_hp)						
	if ( s_4t16 ) or ( s_dp ) then
		hpForUse = 3
	end
	if hpForUse > 0 then
		if ( s_efg ) then
			c_efHaO, c_efHoT = GetEternalFlameHealing(hpForUse) 
		elseif ( s_wog ) then	
			c_wogHaO = GetWordOfGloryHealing(hpForUse)
		end
	end
--]]
--[[
	if rota == 1 or rota % 50 == 0 then 
		print("[",rota,"]  GCD: gcdDur: ",gcdDur," gcdExp: ",gcdExp," c_mHaste: ",c_mHaste," c_time: ",c_time) 
--		print("[",rota,"] INFO: c_time: ",c_time," later: ",later," gcdDur: ",gcdDur," gcdExp: ",gcdExp," c_hp : ",c_hp," c_mHaste: ",c_mHaste) 
	end 
--]]
	return
end

local function GetNextAction()								-- the logic is the same, only the names have been changed :-)

	-- returns the spell id and "code" for the action with the highest priority 
	----: that has the lowest value for its cooldown expiration time (which may be zero) 
	
	local action = ""											
	local cdx = 000.000											
	
	-- set the current db
	db = clcprot.db.profile.rotation

	if s_queue then 
		GetWorkingQueue()
		s_queue = false
	end

	local n = #q

	-- parse once to get cooldowns, return first spell for which the cooldown has expired, if there is one

	for i = 1, n do
		action = actions[q[i]]							-- the value of "action" is an action code specified in the rotation
		if action then 									-- added precaution
			cdx = action.GetCD()						-- the GetCD method calls GetCooldown() for expiration time of CD
			if cdx then 								-- Murphy's Law, general case
				if cdx == 0 then		
					if action.id and q[i] then 			-- added precaution
						if action.id > 0 then 			-- set action.id to zero to exclude an action, e.g., for testing 
							return action.id, q[i]		-- stops GetNextAction(), then s1 (or s2) = action.id, action = q[i]
						end
					else
						print("CLCProt rotation [",rota,"] GetNextAction() 1C: invalid data in actions[q[",i,"]] action.id: ", action.id, " q[i]: ", q[i])
					end	
				end
				action.cdx = cdx 						-- this trick should be documented (no cdx data in actions table) 
			else
				print("CLCProt rotation [",rota,"] GetNextAction() 1B: actions[q[",i,"]] invalid CD expiration for action: ",cdx)
			end
		else
			print("CLCProt rotation [",rota,"] GetNextAction() 1A: actions[q[",i,"]] invalid action: ",actions[q[i]])
		end
	end

	-- parse again, return the spell that expires first		-- executed when all spells are on cooldown
	
	local minQ = 1
	local minCdx = later
	for i = 1, n do
		local action = actions[q[i]]
		if action then
			if action.cdx then 
				if action.cdx < minCdx then					
					minCdx = action.cdx
					minQ = i
				end
			else
				print("CLCProt rotation [",rota,"] GetNextAction() 2B: invalid action.cdx from actions[q[", i, "]]: ",action.cdx)
			end 
		else
			print("CLCProt rotation [",rota,"] GetNextAction() 2A: invalid action from actions[q[", i, "]]: ",action)
		end
	end 
	return actions[q[minQ]].id, q[minQ]						-- the first will be s1, the second will be local action
end

function clcprot.ProtRotation()								-- prior source with comments and additions

	rota = rota + 1											-- rotation count 

	-- set the current db
	db = clcprot.db.profile.rotation

	GetStatus()							-- get current time, buff expiration times, flags, HP, haste and other variables 

--	s_s1 = true							-- debugging - whether skill chosen is for s1
	s1 = 0								-- valid to compare s1 to action spell Id, but beware whether spell Id is zero
	local action1 = "aaa"
	s1, action1 = GetNextAction()		-- s1 is the action's spell ID number, action is a string such as "cons", "sotr", etc.
	actions[action1].UpdateStatus()		-- changes the value of c_time & tracks number of HP charges and of BoG stacked

	GetUpdates()						-- updates time remaining on buffs and updates flags
  	
--	s_s1 = false						-- bebugging - whether skill chosen is for s1
	s2 = 0
	local action2 = "zzz"
	s2, action2 = GetNextAction()

	-- set flag(s) when an AoE spell targeted with a mouse is in s1 and/or s2
	if s1 == lhId or (( s_goc ) and s1 == consId) then s1_twm = true end
	if s2 == lhId or (( s_goc ) and s2 == consId) then s2_twm = true end

	return s1, s2											-- return them to clcprot:CheckQueueProt()
end

--- END OF FILE ---------------------------------------------------------------=