-- CLCProt 4.0.00.003 beta .03 for WoW Warlords of Draenor 6.0.3 (pre-patch)
-- by Adeya, player of Paladin DyeimiAdeya currently on Garona_US
--
--    The CLCProt 4.x files include "YOU MUST READ ME NOW" (.rtf), which has  
-- that name because it is true. :-) It is in the same folder as clcprot.lua.
--
--   CLCProt 4.x is the successor of "CLCProt for WoW Mists of Pandaria 5.4".
-- It is updated for WoW Warlords of Draenor 6.x, to include its new features 
-- and many changes which have been documented by Blizzard Entertainment and 
-- by fan websites such as Wowhead, Wowpedia, Maintankadin, and Elitist Jerks.
--
--  See also the comments at the beginning of rotation.lua.
--
-- COURTESY NOTICE: the Lua scripting language is Copyright © 1994–2014 Lua.org, 
-- PUC-Rio. The Lua license is available at http://www.lua.org/license.html
--
-------------------------------------------------------------------------------=

local _, trueclass = UnitClass("player")
if trueclass ~= "PALADIN" then return end					--dA: prior source, also in clcprot.lua (loaded first)

clcprot.optionsLoaded = true

local MAX_AURAS = 20

local db = clcprot.db.profile
local xdb = clcprot.db.profile.rotation
local root

local strataLevels = {
	"BACKGROUND",
	"LOW",
	"MEDIUM",
	"HIGH",
	"DIALOG",
	"FULLSCREEN",
	"FULLSCREEN_DIALOG",
	"TOOLTIP",
}

local anchorPoints = {
	CENTER = "CENTER",
	TOP = "TOP",
	BOTTOM = "BOTTOM",
	LEFT = "LEFT",
	RIGHT = "RIGHT",
	TOPLEFT = "TOPLEFT",
	TOPRIGHT = "TOPRIGHT",
	BOTTOMLEFT = "BOTTOMLEFT",
	BOTTOMRIGHT = "BOTTOMRIGHT"
}
local execList = {											--dA: shortened strings to fit available space
	AuraButtonExecNone = "None",
	AuraButtonExecSkillVisibleAlways = "Skill always shown",
	AuraButtonExecSkillVisibleNoCooldown = "Skill shown when ready",
	AuraButtonExecSkillVisibleOnCooldown = "Skill shown when not ready",
	AuraButtonExecItemVisibleAlways = "OnUse item always shown",
	AuraButtonExecItemVisibleNoCooldown = "OnUse item shown when usable",
	AuraButtonExecGenericBuff = "Generic buff",
	AuraButtonExecGenericDebuff = "Generic debuff",
	AuraButtonExecPlayerMissingBuff = "Buff missing on player",
	AuraButtonExecICDItem = "ICD Proc",
}
local skillButtonNames = { "Main skill", "Secondary skill" }

-- index lookup for aura buttons
local ilt = {}
for i = 1, MAX_AURAS do
	ilt["aura" .. i] = i
end

-- aura buttons get/set functions
local abgs = {}

function abgs:UpdateAll()
	clcprot:UpdateEnabledAuraButtons()
	clcprot:UpdateAuraButtonsCooldown()
	clcprot:AuraButtonUpdateICD()
	clcprot:AuraButtonResetTextures()
end

-- enabled toggle
function abgs:EnabledGet()
	local i = ilt[self[2]]
	
	return db.auras[i].enabled
end
function abgs:EnabledSet(val)
	local i = ilt[self[2]]
	
	clcprot.temp = info
	if db.auras[i].data.spell == "" then
		val = false
		print("Not a valid spell name/id or buff name!")
	end
	db.auras[i].enabled = val
	if not val then clcprot:AuraButtonHide(i) end
	abgs:UpdateAll()
end

-- id/name field
function abgs:SpellGet()
	local i = ilt[self[2]]
	
	-- special case for items since link is used instead of name
	if (db.auras[i].data.exec == "AuraButtonExecItemVisibleAlways") or (db.auras[i].data.exec == "AuraButtonExecItemVisibleNoCooldown") then
		return db.auras[i].data.spell
	elseif db.auras[i].data.exec == "AuraButtonExecICDItem" then
		return GetSpellInfo(db.auras[i].data.spell)
	end
	return db.auras[i].data.spell
end

function abgs:SpellSet(val)
	local i = ilt[self[2]]
	
	-- skill
	if (db.auras[i].data.exec == "AuraButtonExecSkillVisibleAlways") or (db.auras[i].data.exec == "AuraButtonExecSkillVisibleNoCooldown") or (db.auras[i].data.exec == "AuraButtonExecSkillVisibleOnCooldown") then
		local name = GetSpellInfo(val)
		if name then
			db.auras[i].data.spell = name
		else
			db.auras[i].data.spell = ""
			db.auras[i].enabled = false
			clcprot:AuraButtonHide(i)
			print("Not a valid spell name or id !")
		end
	-- item
	elseif (db.auras[i].data.exec == "AuraButtonExecItemVisibleAlways") or (db.auras[i].data.exec == "AuraButtonExecItemVisibleNoCooldown") then
		local name, link = GetItemInfo(val)
		if name then
			db.auras[i].data.spell = val
		else
			db.auras[i].data.spell = ""
			db.auras[i].enabled = false
			clcprot:AuraButtonHide(i)
			print("Not a valid item name or id !")
		end
	-- icd stuff
	elseif (db.auras[i].data.exec == "AuraButtonExecICDItem") then
		local tid = tonumber(val)
		local name = GetSpellInfo(tid)
		if name then
			db.auras[i].data.spell = tid
		else
			db.auras[i].data.spell = ""
			db.auras[i].enabled = false
			clcprot:AuraButtonHide(i)
			print("Not a valid spell id!")
		end
	else
		db.auras[i].data.spell = val
	end
	
	abgs:UpdateAll()
end

-- type select
function abgs:ExecGet()
	local i = ilt[self[2]]
	
	return db.auras[i].data.exec
end

function abgs:ExecSet(val)
	local i = ilt[self[2]]
	local aura = db.auras[i]
	
	-- reset every other setting when this is changed
	aura.enabled = false
	aura.data.spell = ""
	aura.data.unit = ""
	aura.data.byPlayer = false
	clcprot:AuraButtonHide(i)
	
	aura.data.exec = val
	
	abgs:UpdateAll()
end

-- target field
function abgs:UnitGet()
	local i = ilt[self[2]]
	
	return db.auras[i].data.unit
end

function abgs:UnitSet(val)
	local i = ilt[self[2]]
	
	db.auras[i].data.unit = val
	abgs:UpdateAll()
end

-- cast by player toggle
function abgs:ByPlayerGet()
	local i = ilt[self[2]]
	
	return db.auras[i].data.byPlayer
end

function abgs:ByPlayerSet(val)
	local i = ilt[self[2]]
	
	db.auras[i].data.byPlayer = val
	abgs:UpdateAll()
end

local function RotationGet(info)
--	local xdb = clcprot.db.profile.rotation
	return xdb[info[#info]]
end

local function RotationSet(info, val)
--	local xdb = clcprot.db.profile.rotation
	xdb[info[#info]] = val

	if info[#info] == "prio" or info[#info] == "prioAlt" then 
		clcprot.RR_UpdateQueue()
	end
end

local tx = {}
for k, v in pairs(clcprot.RR_actions) do
	table.insert(tx, format("\n%s - %s", k, v.info))
end
table.sort(tx)
--local prioInfo = "Legend3:\n" .. table.concat(tx)
local prioInfo = "List of Actions\n" .. table.concat(tx)		--dA: this name is more informative

local options = 
{
	type = "group",
	name = "CLCProt",
	args = 
	{
		global = 
		{
			type = "group",
			name = "Global",
			order = 1,
			args = 
			{
			 -- lock frame
				lock = 
				{
					order = 1,
					width = "full",
					type = "toggle",
					name = "Lock Frame",
					get = function(info) return clcprot.locked end,
					set = function(info, val)
						clcprot:ToggleLock()
					end,
				},
				show = 
				{
					order = 10,
					type = "select",
					name = "Show",
					get = function(info) return db.show end,
					set = function(info, val)
						db.show = val
						clcprot:UpdateShowMethod()
					end,
					values = { always = "Always", combat = "In Combat", valid = "Valid Target", boss = "Boss" }
				},
				__strata = 
				{
					order = 15,
					type = "header",
					name = "",
				},
				____strata = 
				{
					order = 16,
					type = "description",
					name = "|cffff0000WARNING|cffffffff Changing Strata value will automatically reload your UI."
				},
				strata = 
				{
					order = 17,
					type = "select",
					name = "Frame Strata",
					get = function(info) return db.strata end,
					set = function(info, val)
						db.strata = val
						ReloadUI()
					end,
					values = strataLevels,
				},
				
				-- full disable toggle
				__fulldisable = 
				{
					order = 20,
					type = "header",
					name = "",
				},
				fullDisable = 
				{
					order = 21,
					width = "full",
					type = "toggle",
					name = "Addon disabled",
					get = function(info) return db.fullDisable end,
					set = function(info, val) clcprot:FullDisableToggle() end,
				},
			},
		},
							
		appearance = 
		{
			order = 10,
			name = "Appearance",
			type = "group",
			args = 
			{
				__buttonAspect = 
				{
					type = "header",
					name = "Button Aspect",
					order = 2,
				},
				zoomIcons = 
				{
					order = 3,
					type = "toggle",
					name = "Zoomed icons",
					get = function(info) return db.zoomIcons end,
					set = function(info, val)
						db.zoomIcons = val
						clcprot:UpdateSkillButtonsLayout()
						clcprot:UpdateAuraButtonsLayout()
					end,
				},
				noBorder = 
				{
					order = 4,
					type = "toggle",
					name = "Hide border",
					get = function(info) return db.noBorder end,
					set = function(info, val)
						db.noBorder = val
						clcprot:UpdateSkillButtonsLayout()
						clcprot:UpdateAuraButtonsLayout()
					end,
				},
				borderColor = 
				{
					order = 5,
					type = "color",
					name = "Border color",
					hasAlpha = true,
					get = function(info) return unpack(db.borderColor) end,
					set = function(info, r, g, b, a)
						db.borderColor = {r, g, b, a}
						clcprot:UpdateSkillButtonsLayout()
						clcprot:UpdateAuraButtonsLayout()
					end,
				},
				borderType = 
				{
					order = 6,
					type = "select",
					name = "Border type",
					get = function(info) return db.borderType end,
					set = function(info, val)
						db.borderType = val
						clcprot:UpdateSkillButtonsLayout()
						clcprot:UpdateAuraButtonsLayout()
					end,
					values = { "Light", "Medium", "Heavy" }
				},
				grayOOM = 
				{
					order = 7,
					type = "toggle",
					name = "Gray when OOM",
					get = function(info) return db.grayOOM end,
					set = function(info, val)
						db.grayOOM = val
						clcprot:ResetButtonVertexColor()
					end,
				},
				
				__hudAspect = 
				{
					type = "header",
					name = "HUD Aspect",
					order = 10,
				},
				scale = 
				{
					order = 11,
					type = "range",
					name = "Scale",
					min = 0.01,
					max = 3,
					step = 0.01,
					get = function(info) return db.scale end,
					set = function(info, val)
						db.scale = val
						clcprot:UpdateFrameSettings()
					end,
				},
				alpha = 
				{
					order = 12,
					type = "range",
					name = "Alpha",
					min = 0,
					max = 1,
					step = 0.001,
					get = function(info) return db.alpha end,
					set = function(info, val)
						db.alpha = val
						clcprot:UpdateFrameSettings()
					end,
				},
				
				_hudPosition = 
				{
					type = "header",
					name = "HUD Position",
					order = 13,
				},
				x = 
				{
					order = 20,
					type = "range",
					name = "X",
					min = 0,
					max = 5000,
					step = 21,
					get = function(info) return db.x end,
					set = function(info, val)
						db.x = val
						clcprot:UpdateFrameSettings()
					end,
				},
				y = 
				{
					order = 22,
					type = "range",
					name = "Y",
					min = 0,
					max = 3000,
					step = 1,
					get = function(info) return db.y end,
					set = function(info, val)
						db.y = val
						clcprot:UpdateFrameSettings()
					end,
				},
				align = 
				{
					order = 23,
					type = "execute",
					name = "Center Horizontally",
					func = function()
						clcprot:CenterHorizontally()
					end,
				},
				
				__icd = 
				{
					order = 50,
					type = "header",
					name = "ICD Visibility",
				},
				____icd = 
				{
					order = 51,
					type = "description",
					name = "Controls the way ICD Aura Buttons are displayed while the proc is ready or on cooldown.",
				},
				icdReady = 
				{
					order = 52,
					type = "select",
					name = "Ready",
					values = { [1] = "Visible", [2] = "Faded", [3] = "Invisible" },
					get = function(info) return db.icd.visibility.ready end,
					set = function(info, val)
						db.icd.visibility.ready = val
					end,
				},
				icdCooldown = 
				{
					order = 53,
					type = "select",
					name = "On cooldown",
					values = { [1] = "Visible", [2] = "Faded", [3] = "Invisible" },
					get = function(info) return db.icd.visibility.cd end,
					set = function(info, val)
						db.icd.visibility.cd = val
					end,
				}
			},
		},
	
		behavior = 
		{
			order = 15,
			name = "Behavior",
			type = "group",
			args = 
			{
				__updateRates = 
				{
					order = 1,
					type = "header",
					name = "Updates per Second",
				},
				ups = 
				{
					order = 5,
					type = "range",
					name = "FCFS Detection",
					min = 1,
					max = 100,
					step = 1,
					get = function(info) return db.updatesPerSecond end,
					set = function(info, val)
						db.updatesPerSecond = val
						clcprot.scanFrequency = 1 / val
					end,
				},
				upsAuras = 
				{
					order = 6,
					type = "range",
					name = "Aura Detection",
					min = 1,
					max = 100,
					step = 1,
					get = function(info) return db.updatesPerSecondAuras end,
					set = function(info, val)
						db.updatesPerSecondAuras = val
						clcprot.scanFrequencyAuras = 1 / val
					end,
				},
			
				__rangePerSkill = 
				{
					order = 15,
					type = "header",
					name = "Range Display",
				},
				____rangePerSkill = 
				{
					order = 16,
					type = "description",
					name = "By default the addon checks if you are in melee range and colors both main and secondary skills if not. This option allows you to display the range of the actual skills displayed.",
				},
				rangePerSkill = 
				{
					order = 17,
		 			type = "toggle",
					width = "full",
					name = "Check range for each skill",
					get = function(info) return db.rangePerSkill end,
					set = function(info, val) db.rangePerSkill = val end,
				},
			},
		},
		--dA: the rotation section now has seven tabs: 
		----: (1) priority sequences and optional timing adjustments 
		----: (2) 'primary options' pertaining to SotR(BoG), EF, WoG, SS, WB and Cons
		----: (3) settings that apply to SotR, EF, and WoG ordinarily
		----: (4) settings that apply to SotR, EF, and WoG during Divine Purpose
		----: (5) settings that apply to SotR, EF, and WoG during Holy Avenger
        ----: (6) settings that apply to SotR, EF, and WoG during Sanctified Wrath
		----: (7) settings that apply while the armor Tier 16 4-piece set bonus is in effect
        ----:
		rotation = 
		{
		    order = 20,
			name = "Rotation",
			type = "group", childGroups = "tab", 
			args = 
			{
				tabPriority = 
				{
					order = 1, type = "group", name = "Priority", 
					args = 
					{
						igPrio = 
						{
							order = 2, type = "group", inline = true, name = "",
							args = 
							{
								info = 
								{ 
									order = 1, type = "description", name = prioInfo,
								},
								normalPrio = 
								{ 
									order = 3, type = "group", inline = true, name = "Normal Priority",
									args = 
									{ --10--
										note = 
										{ 
											order = 1, type = "description", name = "Select talents & choose glyphs |cffff0000before|cffffffff you enter any action priority sequence.", 
										},
										__space = 
										{ 
											order = 2, type = "description", name = "",
										},
										prio = 
										{
											order = 3, type = "input", width = "full", name = "",
											get = RotationGet, set = RotationSet,
										},
										infoCMD = 
										{
											order = 5, type = "description", name = "Sample command line usage: /clcprotlp efg ss wog sor cs j hwsw asgc hwf as how hw cons",
										},
									},
								},
								AltPrio = 
								{ 
									order = 5, type = "group", inline = true, name = "Alternate Priority",
									args = 
									{
										defaultPrioAlt = 
										{
											order = 1, type = "toggle", name = "Use as default",
											get = function(info) return xdb.defaultPrioAlt end,		--dA: db. must be xdb. instead
											set = function(info, val) clcprot:ToggleRotation() xdb.defaultPrioAlt = val end,
										},					
										binding = 
										{
											order = 2, type = "keybinding", name = "Keybinding",
											get = function() return GetBindingKey("TOGGLEROTATION")	end,
											set = function(info, v) SetBinding(v, "TOGGLEROTATION")	SaveBindings(GetCurrentBindingSet()) end,
										},
										note = 
										{ 
											order = 3, type = "description", name = "Select talents & choose glyphs |cffff0000before|cffffffff you enter any action priority sequence.", 
										},
										spacing = 
										{ 
											order = 4, type = "description", name = "",
										},
										prioAlt = 
										{
											order = 5, type = "input", width = "full", name = "",
											get = RotationGet, set = RotationSet,
										},
										infoCMD = 
										{
											order = 7, type = "description", name = "Sample command line usage: /clcprotlpalt efg ss wog sor hotr j hwsw asgc hwf as how cons hw",
										},
									},
								},						
								spacing6 = 
								{
									order = 6, type = "description", name = "",
								},
								disclaimer = 
								{
									order = 7, type = "description", name = "|cffffaaaaTHE SEQUENCES ABOVE ARE JUST EXAMPLES, YOU MUST DILIGENTLY COMPOSE ONES THAT YOU USE!|cffffffff",
								},
								spacing8 = 
								{
									order = 8, type = "description", name = "",
								},
							},
						},
						igTiming = 
						{
							order = 4, type = "group", inline = true, name = "",
							args = 
							{
								__spacing1 =
								{
									order = 1, type = "description", name = "", 
								},
								__heading1 = 
								{
									order = 2, type = "header",	name = "Optional: Timing Settings",
								},
								__adage =
								{
									order = 4, type = "description", name = "|cffff0000                                          Fools rush in where angels fear to tread. --Milton :-)|cffffffff",
								},
								__spacing2 =
								{
									order = 5, type = "description", name = "", 
								},
								__beware =
								{
									order = 7, type = "description", name = "BEWARE: you must use Simulation Craft (SimC) or a similar tool to determine whether any option is needed, and, if so, which option(s) to use and the value to set each one.",
								},
								__spacing3 =
								{
									order = 9, type = "description", name = "", 
								},
								prioTimes = 
								{
									order = 20, type = "group", inline = true, name = "Action Priority Times",
									args = 
									{
										timeInfo =
										{ 
											order = 1, type = "description", name = "Used to evaluate the expiration time returned by GetCD() while parsing for the secondary skill (s2) when one of these actions has already been selected as the primary skill (s1). Increment is 0.01 sec.",
										},
										spacing1 =
										{
											order = 2, type = "description", name = "", 
										},
										minGCDinfo =
										{
											order = 10, type = "description", name = "Minimum Global Cooldown plus action priority time:",	
										},
										efcPrio =
										{
											order = 11, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "efc",
											get = RotationGet, set = RotationSet,
										},
										efgPrio =
										{
											order = 12, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "efg",
											get = RotationGet, set = RotationSet,
										},
										sotrxxPrio =
										{
											order = 16, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "sordp / sorha / sorsw",
											get = RotationGet, set = RotationSet,
										},
										sotrPrio =
										{
											order = 17, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "sor",
											get = RotationGet, set = RotationSet,
										},
--[[	
-- ToDo: needs further consideration & testing 
										ssPrio =
										{
											order = 18, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "ss",
											get = RotationGet, set = RotationSet,
										},
--]]
										wogxxPrio =
										{
											order = 19, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "wogdp / wogha / wogsw",
											get = RotationGet, set = RotationSet,
										},
										wogPrio = 
										{
											order = 20, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "wog",
											get = RotationGet, set = RotationSet,
										},
										spacing2 =
										{
											order = 30, type = "description", name = "", 
										},
										maxGCDinfo =
										{
											order = 40, type = "description", name = "Maximum Global Cooldown plus action priority time:",	
										},
										asPrio = 
										{
											order = 41, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "as",
											get = RotationGet, set = RotationSet,
										},
										asgcPrio = 
										{
											order = 42, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "asgc",
											get = RotationGet, set = RotationSet,
										},
										csPrio = 
										{
											order = 43, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "cs",
											get = RotationGet, set = RotationSet,
										},
										hotrPrio = 
										{
											order = 44, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "hotr",
											get = RotationGet, set = RotationSet,
										},			
										howPrio = 
										{
											order = 45, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "how",
											get = RotationGet, set = RotationSet,
										},
										hwPrio = 
										{
											order = 46, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "hw",
											get = RotationGet, set = RotationSet,
										},
										hwfPrio = 
										{
											order = 47, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "hwf",
											get = RotationGet, set = RotationSet,
										},
										hwswPrio = 
										{
											order = 48, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "hwsw",
											get = RotationGet, set = RotationSet,
										},
										jPrio = 
										{
											order = 49, type = "range", min = 0.11, max = 1.49, step = 0.01, name = "j",
											get = RotationGet, set = RotationSet,
										},
									},
								},
							},
						},
					},
				},
				tabPrimaryOptions = 
				{ 
					order = 2, type = "group", name = "Primary Options", 
					args = 
					{
						spacing0 = 
						{
							order = 1, type = "description", name = "",
						},
--[[					--dA: no comment at the top seem necessary or desirable at this time :-)
						spacing1 = 
						{
							order = 2, type = "description", name = "",
						},
						spacing2 =  
						{
							order = 3, type = "description", name = "",
						},
--]]
						igBoG =
						{ 
							order = 5, type = "group", inline = true, name = "Bastion of Glory Refresh",
							args = 
							{					
								BoGinfo = 
								{
									order = 1, type = "description", name = "These conditions always govern whether to display Shield of the Righteous in order to refresh the duration of the Bastion of Glory stack when at least 3 HP charges are available (or a DP free cast):",
								},
								_spacing = 
								{
									order = 2, type = "description", name = "",
								},
								sotrBoGxp = 
								{
									order = 5, type = "range", min = 3, max = 12, step = 0.1, name = "Refresh BoG @__ sec left",
									get = RotationGet, set = RotationSet,
								},
								sotrBoGstk =
								{
									order = 7, type = "range", min = 1, max = 5, step = 1, name = "Refresh BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igEFCexp =
						{ 
							order = 7, type = "group", inline = true, name = "Activate Continuous Eternal Flame:  efc",
							args = 
							{					
								EFCexpInfo = 
								{
									order = 1, type = "description", name = "Specify the minimum charges of Holy Power to use for casting Continuous Eternal Flame when it is not active, regardless of the BoG stack (if any).",
								},
								_spacing = 
								{
									order = 2, type = "description", name = "",
								},
								efcxpHP = 
								{
									order = 5, type = "range", min = 1, max = 3, step = 1, name = "Use HP >= __ to activate EF",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igEFGalternate =
						{ 
							order = 9, type = "group", inline = true, name = "Glorious Eternal Flame Alternative:  efg",
							args = 
							{					
								EFGaltInfo = 
								{
									order = 1, type = "description", name = "Apply the percentage of the paladin's maximum Health as a goal toward which Health will be increased (instead of as a trigger) to show EF. Note that your configured HP charge and/or BoG stack amounts also affect whether EF is shown.",
								},
								_spacing = 
								{
									order = 2, type = "description", name = "",
								},
								useEFGalt =
								{
									order = 3, type = "toggle", name = " Use efg alternative  instead of the default.",
									get = function(info) return xdb.useEFGalt end,		--dA: db. must be xdb. instead
									set = function(info, val) clcprot:Toggle_useEFGalt() xdb.useEFGalt = val end,
								},
							},
						},
						igWoGalternate =
						{ 
							order = 11, type = "group", inline = true, name = "Word of Glory Alternative:  wog, wogdp, wogha, wogsw",
							args = 
							{					
								WoGaltInfo = 
								{
									order = 1, type = "description", name = "Apply the percentage of the paladin's maximum Health as a goal toward which Health will be increased (instead of as a trigger) to show EF. Note that your configured HP charge and/or BoG stack amounts also affect whether EF is shown.",
								},
								_spacing = 
								{
									order = 2, type = "description", name = "",
								},
								useWoGalt =
								{
									order = 3, type = "toggle", name = "Use wog alternative  instead of the default.",
									get = function(info) return xdb.useWoGalt end,		--dA: db. must be xdb. instead
									set = function(info, val) clcprot:Toggle_useWoGalt() xdb.useWoGalt = val end,
								},
							},
						},
						igSShield =
						{ 
							order = 13, type = "group", inline = true, name = "Sacred Shield:  ss",
							args = 
							{					
								ssRefresh = 
								{
									order = 1, type = "range", min = 2.5, max = 12.0, step = 0.1, name = "Refresh ss @ _ sec left",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igConsecration =
						{
							order = 17, type = "group", inline = true, name = "Consecration:  cons",
							args = 
							{
								consMana = 
								{
									order = 1, type = "range", min = 7.1, max = 100, step = 0.1, name = "Min mana for cons = __%",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
				tabSettings1 = 
				{ 
					order = 3, type = "group", name = "Ordinary Settings", 
					args = 
					{
						spacing0 = 
						{
							order = 1, type = "description", name = "",
						},
						ordinaryInfo =
						{
							order = 4, type = "description", name = "These settings apply while Divine Purpose or Holy Avenger is -NOT- in effect. Settings for Sanctified Wrath are in effect at all times while the talent is selected.", 
						},
						spacing2 = 
						{
							order = 5, type = "description", name = "",
						},
						igSoR =
						{
							order = 6, type = "group", inline = true, name = "Shield of the Righteous:  sotr",								
							args =	
							{	
								sotrHP =
								{
									order = 1, type = "range", min = 3, max = 5, step = 1, name = "Use sotr if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								spacing1 = 
								{ 
									order = 7, type = "description", name = "",
								},
								spacing2 = 
								{ 
									order = 8, type = "description", name = "",
								},
								sorBoGnote =
								{
									order = 9, type = "description", name = "See also:  Primary Options - Bastion of Glory Refresh",
								},
							},
						},
						igefc =
						{
							order = 8, type = "group", inline = true, name = "Continuous Eternal Flame:  efc",
							args =	
							{	
								efcHP = 
								{
									order = 1, type = "range", min = 1, max = 5, step = 1, name = "Use efc if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efcRefresh = 
								{
									order = 3, type = "range", min = 0, max = 20.0, step = 0.1, name = "Refresh efc @__ sec left",
									get = RotationGet, set = RotationSet,
								},						
								efcBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efc if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efcBoGinfo =
								{
									order = 6, type = "description", name = "BoG Expiration ____________",
								},
								efcBoGhp = 
								{
									order = 7, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efcBoGxp = 
								{
									order = 9, type = "range", min = 3, max = 12.0, step = 0.1, name = "Use BoG if sec left <=__",
									get = RotationGet, set = RotationSet,
								},
								efcBoGstk =
								{
									order = 11, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefg = 
						{
							order = 10, type = "group", inline = true, name = "Glorious Eternal Flame:  efg",
							args =	
							{	
								efgDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								efgHP = 
								{
									order = 3, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efgBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efgHealth = 
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use efg if Health <= __%",
									get = RotationGet, set = RotationSet,
								},	
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								efgAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								efgAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efgAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efgAltHealth = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igWoG = 
						{
							order = 12, type = "group", inline = true, name = "Word of Glory:  wog",
							args = 
							{
								wogDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								wogHp = 
								{
									order = 2, type = "range", min = 1, max = 5, step = 1, name = "Use wog if HP >= __",
									get = RotationGet, set = RotationSet,
								},					
								wogBasStack = 
								{
									order = 3, type = "range", min = 0, max = 5, step = 1, name = "Use wog if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								wogPercent = 
								{
									order = 5, type = "range", min = 21, max = 100, step = 1, name = "Use wog if Health <= __%",
									get = RotationGet, set = RotationSet,							
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								wogAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								wogAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								wogAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								wogAltPct = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
				tabSettings2 = 
				{ 
					order = 4, type = "group", name = "Divine Purpose", 
					args = 
					{
						spacing0 = 
						{
							order = 21, type = "description", name = "",
						},
						dpInfo = 
						{
							order = 23, type = "description", name = "All settings on this tab refer to and apply to using the DP 'free cast' proc. An action consumes zero HP when a DP proc is available but casts as if 3 HP were used.",
						},
						igSoRDivPurp =
						{
							order = 26, type = "group", inline = true, name = "Shield of the Righteous:  sotr & sordp",								
							args =	
							{	
								sotrdpHP =
								{
									order = 1, type = "range", min = 0, max = 5, step = 1, name = "Use if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								sotr_dptr =
								{
									order = 5, type = "range", min = 2.10, max = 6.00, step = 0.01, name = "Use if DP sec. left <= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefcDivPurp =
						{ 
							order = 28, type = "group", inline = true, name = "Continuous Eternal Flame:  efc",
							args = 
							{
								efcdpRefresh = 
								{
									order = 3, type = "range", min = 0, max = 20.0, step = 0.1, name = "Refresh efc @ __ sec. left",
									get = RotationGet, set = RotationSet,
								},						
								efcdpBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},		
								efcdpBoGinfo =
								{
									order = 6, type = "description", name = "BoG Expiration ____________",
								},
								efcdpBoGxp = 
								{
									order = 9, type = "range", min = 3, max = 12, step = 0.1, name = "Use BoG if sec left <=__",
									get = RotationGet, set = RotationSet,
								},
								efcdpBoGstk =
								{
									order = 11, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefgDivPurp =
						{
							order = 30, type = "group", inline = true, name = "Glorious Eternal Flame:  efg",
							args =	
							{	
								efgdpDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								efgdpBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efgdpHealth =
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use if Health <= __%",
									get = RotationGet, set = RotationSet,
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								efgdpAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								efgdpAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efgdpAltHealth = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igWoGDivPurp = 
						{
							order = 32, type = "group", inline = true, name = "Word of Glory:  wog & wogdp",
							args = 
							{
								wogdpDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								wogdpBasStack = 
								{
									order = 3, type = "range", min = 0, max = 5, step = 1, name = "Use if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								wogdpPercent = 
								{
									order = 5, type = "range", min = 21, max = 100, step = 1, name = "Use if Health <= __%",
									get = RotationGet, set = RotationSet,							
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								wogdpAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								wogdpAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								wogdpAltPct = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
				tabSettings3 = 
				{ 
					order = 5, type = "group", name = "Holy Avenger", 
					args = 
					{
						spacing0 = 
						{
							order = 1, type = "description", name = "",
						},
						haInfo = 
						{
							order = 3, type = "description", name = "These settings are applied while Holy Avenger is in effect.",
						},
						spacing1 = 
						{
							order = 1, type = "description", name = "",
						},
						igSoRHolyAvenger =
						{
							order = 6, type = "group", inline = true, name = "Shield of the Righteous:  sotr & sorha",								
							args =	
							{	
								sotrhaHP =
								{
									order = 1, type = "range", min = 3, max = 5, step = 1, name = "Use sotr if HP >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefcHolyAvenger = 
						{ 
							order = 9, type = "group", inline = true, name = "Continuous Eternal Flame:  efc",
							args = 
							{
								efchaHP = 
								{
									order = 1, type = "range", min = 1, max = 5, step = 1, name = "Use efc if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efchaRefresh = 
								{
									order = 3, type = "range", min = 0, max = 20.0, step = 0.1, name = "Refresh efc @ __ sec. left",
									get = RotationGet, set = RotationSet,
								},
								efchaBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efc if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efchaBoGinfo =
								{
									order = 6, type = "description", name = "BoG Expiration ____________",
								},
								efchaBoGhp = 
								{
									order = 7, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efchaBoGxp = 
								{
									order = 9, type = "range", min = 3, max = 12.0, step = 0.1, name = "Use BoG if sec left <=__",
									get = RotationGet, set = RotationSet,
								},
								efchaBoGstk =
								{
									order = 11, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefgHolyAvenger =
						{
							order = 11, type = "group", inline = true, name = "Glorious Eternal Flame:  efg",								
							args =	
							{	
								efghaDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								efghaHP = 
								{
									order = 3, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efghaBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								efghaHealth =
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use efg if Health <= __%",
									get = RotationGet, set = RotationSet,
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								efghaAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								efghaAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efghaAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efghaAltHealth = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igWoGHolyAvenger = 
						{
							order = 13, type = "group", inline = true, name = "Word of Glory:  wog & wogha",
							args = 
							{
								woghaDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								woghaHp =
								{
									order = 3, type = "range", min = 1, max = 5, step = 1, name = "Use wog if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								woghaBasStack = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use wog if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								woghaPercent = 
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use wog if Health <= __%",
									get = RotationGet, set = RotationSet,							
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								woghaAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								woghaAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use wog if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								woghaAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use wog if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								woghaAltPct = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
				tabSettings4 = 
				{ 
					order = 6, type = "group", name = "Sanctified Wrath", 
					args = 
					{
						spacing0 = 
						{
							order = 1, type = "description", name = "",
						},
						swInfo = 
						{
							order = 2, type = "description", name = "These settings are applied at all times while the Sanctified Wrath talent is selected.",
						},
						spacing1 = 
						{
							order = 3, type = "description", name = "",
						},
						igSoRSanctWrath =
						{
							order = 26, type = "group", inline = true, name = "Shield of the Righteous: sotr & sorsw",								
							args =	
							{	
								sotrswHP =
								{
									order = 1, type = "range", min = 3, max = 5, step = 1, name = "Use SotR if HP >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefcSanctWrath = 
						{ 
							order = 28, type = "group", inline = true, name = "Continuous Eternal Flame:  efc",
							args = 
							{
								efcswHP = 
								{
									order = 1, type = "range", min = 1, max = 5, step = 1, name = "Use efc if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efcswRefresh = 
								{
									order = 3, type = "range", min = 0, max = 20.0, step = 0.1, name = "Refresh efc @ __ sec. left",
									get = RotationGet, set = RotationSet,
								},
								efcswBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efc if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								efcswBoGinfo =
								{
									order = 6, type = "description", name = "BoG Expiration ____________",
								},
								efcswBoGhp = 
								{
									order = 7, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efcswBoGxp = 
								{
									order = 9, type = "range", min = 3, max = 12.0, step = 0.1, name = "Use BoG if sec left <=__",
									get = RotationGet, set = RotationSet,
								},
								efcswBoGstk =
								{
									order = 11, type = "range", min = 1, max = 5, step = 1, name = "Use BoG if stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igefgSanctWrath =
						{
							order = 30, type = "group", inline = true, name = "Glorious Eternal Flame:  efg",								
							args =	
							{	
								efgswDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								efgswHP = 
								{
									order = 3, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efgswBoG = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								efgswHealth =
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use efg if Health <= __%",
									get = RotationGet, set = RotationSet,
								},
								spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								efgswAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								efgswAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use efg if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								efgswAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efgswAltHealth = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						igWoGSanctWrath = 
						{
							order = 32, type = "group", inline = true, name = "Word of Glory: wog & wogsw",
							args = 
							{
								wogswDefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								wogswHp =
								{
									order = 3, type = "range", min = 1, max = 5, step = 1, name = "Use wog if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								wogswBasStack = 
								{
									order = 5, type = "range", min = 0, max = 5, step = 1, name = "Use wog if BoG stack >=__",
									get = RotationGet, set = RotationSet,
								},						
								wogswPercent = 
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use wog if Health <= __%",
									get = RotationGet, set = RotationSet,							
								},
									spacing1 =
								{
									order = 9, type = "description", name = "",
								},
								wogswAlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								wogswAltHP = 
								{
									order = 13, type = "range", min = 1, max = 5, step = 1, name = "Use wog if HP >= __",
									get = RotationGet, set = RotationSet,
								},
								wogswAltBoG = 
								{
									order = 15, type = "range", min = 0, max = 5, step = 1, name = "Use wog if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								wogswAltPct = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
				tabSettings5 = 
				{ 
					order = 7, type = "group", name = "Armor Tier Sets",
					args = 
					{
						__spacing1 =
						{
							order = 1, type = "description", name = "", 
						},
						__heading1 = 
						{
							order = 3, type = "header",	name = "Tier 16 4-Piece Set Bonus",
						},
						__spacing2 =
						{
							order = 5, type = "description", name = "", 
						},
						__Tier16_4pieceInfo = 
						{
							order = 7, type = "description", name = "When the paladin has 4 pieces of the Tier 16 armor set equipped:  while 3 or more BoG are available, the next use of EF or WoG costs zero HP but casts as if 3 HP are consumed. Note that all of your other configurations of HP, BoG and Health for efc, efg, and wog govern until 3 BoG become available, then these settings are applied ordinarily and during Holy Avenger, Sanctified Wrath and Divine Purpose respectively.",
						},
						__spacing3 =
						{
							order = 9, type = "description", name = "", 
						},
						ig4T16_ShieldOfTheRighteous =
						{
							order = 11, type = "group", inline = true, name = "Shield of the Righteous :  sotr",
							args =	
							{	
								sotr4t16HP = 
								{
									order = 5, type = "range", min = 3, max = 5, step = 1, name = "Use SotR if HP >= __",
									get = RotationGet, set = RotationSet,
								},						
							},
						},
						ig4T16_EFcontinuous =
						{
							order = 13, type = "group", inline = true, name = "Continuous Eternal Flame:  efc",
							args =	
							{	
								efc4T16Refresh = 
								{
									order = 3, type = "range", min = 0, max = 12.0, step = 0.1, name = "Refresh efc @ __ sec. left",
									get = RotationGet, set = RotationSet,
								},
								efc4T16BoG = 
								{
									order = 5, type = "range", min = 3, max = 5, step = 1, name = "Use efc if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								efc4T16BoGinfo =
								{
									order = 7, type = "description", name = "BoG Expiration ____________",
								},
								efc4T16BoGxp = 
								{
									order = 9, type = "range", min = 3, max = 12.0, step = 0.1, name = "Use BoG if sec left <=__",
									get = RotationGet, set = RotationSet,
								},
								efc4T16BoGstk =
								{
									order = 11, type = "range", min = 3, max = 5, step = 1, name = "Use BoG if stack >= __",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						ig4T16_EFglorious =
						{
							order = 15, type = "group", inline = true, name = "Glorious Eternal Flame:  efg",
							args =	
							{	
								efg4T16DefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								efg4T16BoG = 
								{
									order = 5, type = "range", min = 3, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},						
								efg4T16Health =
								{
									order = 7, type = "range", min = 21, max = 100, step = 1, name = "Use efg if Health <= __%",
									get = RotationGet, set = RotationSet,
								},
								spacing =
								{
									order = 9, type = "description", name = "",
								},
								efg4T16AlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								efg4T16AltBoG = 
								{
									order = 15, type = "range", min = 3, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								efg4T16AltHealth = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
						ig4T16_WordOfGlory = 
						{
							order = 17, type = "group", inline = true, name = "Word of Glory:  wog, wogdp, wogha, wogsw",
							args = 
							{
								wog4T16DefaultInfo =
								{
									order = 1, type = "description", name = "Default: decreasing health is a trigger.",
								},
								wog4T16BasStack = 
								{
									order = 3, type = "range", min = 3, max = 5, step = 1, name = "Use wog if BoG stack >=__",
									get = RotationGet, set = RotationSet,
								},						
								wog4T16Percent = 
								{
									order = 5, type = "range", min = 21, max = 100, step = 1, name = "Use wog if Health <= __%",
									get = RotationGet, set = RotationSet,							
								},
									spacing =
								{
									order = 9, type = "description", name = "",
								},
								wog4T16AlternateInfo =
								{
									order = 11, type = "description", name = "Alternative: level of health is a goal.",
								},
								wog4T16AltBoG = 
								{
									order = 15, type = "range", min = 3, max = 5, step = 1, name = "Use efg if BoG stack >= __",
									get = RotationGet, set = RotationSet,
								},
								wog4T16AltPct = 
								{
									order = 17, type = "range", min = 21, max = 100, step = 1, name = "Restore to __% Health",
									get = RotationGet, set = RotationSet,
								},
							},
						},
					},
				},
			},
		},
	
		-- aura buttons
		auras = 
		{
			order = 30,
			name = "Aura Buttons",
			type = "group",
			args = 
			{
				____info = 
				{
					order = 1,
					type = "description",
					name = "These are cooldown watchers. You can select a player skill, an item or a buff/debuff (on a valid target) to watch.\nItems and skills only need a valid item/spell id (or name) and the type. Target (the target to scan) and Cast by player (filters or not buffs cast by others) are specific to buffs/debuffs.\nValid targets are the ones that work with /cast [target=name] macros. For example: player, target, focus, raid1, raid1target.\n\nICD Proc:\nYou need to specify a valid proc ID (example: 60229 for Greatness STR proc) Name doesn't work, if the ID is valid it will be replaced by the name after the edit.\nIn the \"Target unit\" field you have to enter the ICD and duration of the proc separated by \":\" (example: for Greatness the value should be 45:15).",
				},
			},
		},
	
		-- layout
		layout = 
		{
			order = 31,
			name = "Layout",
			type = "group",
			args = {},
		},
		
	},
}

-- add main buttons to layout
for i = 1, 2 do
	options.args.layout.args["button" .. i] = 
	{
		order = i,
		name = skillButtonNames[i],
		type = "group",
		args = 
		{
			size = 
			{
				order = 1,
				type = "range",
				name = "Size",
				min = 1,
				max = 300,
				step = 1,
				get = function(info) return db.layout["button" .. i].size end,
				set = function(info, val)
					db.layout["button" .. i].size = val
					clcprot:UpdateSkillButtonsLayout()
				end,
			},
			alpha = 
			{
				order = 2,
				type = "range",
				name = "Alpha",
				min = 0,
				max = 1,
				step = 0.01,
				get = function(info) return db.layout["button" .. i].alpha end,
				set = function(info, val)
					db.layout["button" .. i].alpha = val
					clcprot:UpdateSkillButtonsLayout()
				end,
			},
			anchor = 
			{
				order = 6,
				type = "select",
				name = "Anchor",
				get = function(info) return db.layout["button" .. i].point end,
				set = function(info, val)
					db.layout["button" .. i].point = val
					clcprot:UpdateSkillButtonsLayout()
				end,
				values = anchorPoints,
			},
			anchorTo = 
			{
				order = 6,
				type = "select",
				name = "Anchor To",
				get = function(info) return db.layout["button" .. i].pointParent end,
				set = function(info, val)
					db.layout["button" .. i].pointParent = val
					clcprot:UpdateSkillButtonsLayout()
				end,
				values = anchorPoints,
			},
			x = 
			{
				order = 10,
				type = "range",
				name = "X",
				min = -1000,
				max = 1000,
				step = 1,
				get = function(info) return db.layout["button" .. i].x end,
				set = function(info, val)
					db.layout["button" .. i].x = val
					clcprot:UpdateSkillButtonsLayout()
				end,
			},
			y = 
			{
				order = 11,
				type = "range",
				name = "Y",
				min = -1000,
				max = 1000,
				step = 1,
				get = function(info) return db.layout["button" .. i].y end,
				set = function(info, val)
					db.layout["button" .. i].y = val
					clcprot:UpdateSkillButtonsLayout()
				end,
			},
		},
	}
end
	
-- add the buttons to options
for i = 1, MAX_AURAS do
	-- aura options
	options.args.auras.args["aura" .. i] = 
	{
		order = i + 10,
		type = "group",
		name = "Aura Button " .. i,
		args = 
		{
			enabled = 
			{
				order = 1,
				type = "toggle",
				name = "Enabled",
				get = abgs.EnabledGet,
				set = abgs.EnabledSet,
			},
			spell = 
			{
				order = 5,
				type = "input",
				name = "Spell/item name/id or buff to track",
				get = abgs.SpellGet,
				set = abgs.SpellSet,
			},
			exec = 
			{
				order = 10,
				type = "select",
				name = "Type",
				get = abgs.ExecGet,
				set = abgs.ExecSet,
				values = execList,
			},
			unit = 
			{
				order = 15,
				type = "input",
				name = "Target unit",
				get = abgs.UnitGet,
				set = abgs.UnitSet,
			},
			byPlayer = 
			{
				order = 16,
				type = "toggle",
				name = "Cast by player",
				get = abgs.ByPlayerGet,
				set = abgs.ByPlayerSet,
			},
		},
	}
	
	-- layout
	options.args.layout.args["aura" .. i] = 
	{
		order = 10 + i,
		type = "group",
		name = "Aura Button " .. i,
		args = 
		{
			size = 
			{
				order = 1,
				type = "range",
				name = "Size",
				min = 1,
				max = 300,
				step = 1,
				get = function(info) return db.auras[i].layout.size end,
				set = function(info, val)
					db.auras[i].layout.size = val
					clcprot:UpdateAuraButtonLayout(i)
				end,
			},
			anchor = 
			{
				order = 6,
				type = "select",
				name = "Anchor",
				get = function(info) return db.auras[i].layout.point end,
				set = function(info, val)
					db.auras[i].layout.point = val
					clcprot:UpdateAuraButtonLayout(i)
				end,
				values = anchorPoints,
			},
			anchorTo = 
			{
				order = 6,
				type = "select",
				name = "Anchor To",
				get = function(info) return db.auras[i].layout.pointParent end,
				set = function(info, val)
					db.auras[i].layout.pointParent = val
					clcprot:UpdateAuraButtonLayout(i)
				end,
				values = anchorPoints,
			},
			x = 
			{
				order = 10,
				type = "range",
				name = "X",
				min = -1000,
				max = 1000,
				step = 1,
				get = function(info) return db.auras[i].layout.x end,
				set = function(info, val)
					db.auras[i].layout.x = val
					clcprot:UpdateAuraButtonLayout(i)
				end,
			},
			y = 
			{
				order = 11,
				type = "range",
				name = "Y",
				min = -1000,
				max = 1000,
				step = 1,
				get = function(info) return db.auras[i].layout.y end,
				set = function(info, val)
					db.auras[i].layout.y = val
					clcprot:UpdateAuraButtonLayout(i)
				end,
			},
		},
	}
end

	-- remove the first one we added
for i = 1, #INTERFACEOPTIONS_ADDONCATEGORIES do
	if 	INTERFACEOPTIONS_ADDONCATEGORIES[i]
	and INTERFACEOPTIONS_ADDONCATEGORIES[i].name
	and INTERFACEOPTIONS_ADDONCATEGORIES[i].name == "CLCProt"
	then
		table.remove(INTERFACEOPTIONS_ADDONCATEGORIES, i)
	end
end
	
local AceConfig = LibStub("AceConfig-3.0")
AceConfig:RegisterOptionsTable("CLCProt", options)

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
AceConfigDialog:AddToBlizOptions("CLCProt", "CLCProt", nil, "global")
AceConfigDialog:AddToBlizOptions("CLCProt", "Appearance", "CLCProt", "appearance")
AceConfigDialog:AddToBlizOptions("CLCProt", "Rotation", "CLCProt", "rotation")
AceConfigDialog:AddToBlizOptions("CLCProt", "Behavior", "CLCProt", "behavior")
AceConfigDialog:AddToBlizOptions("CLCProt", "Aura Buttons", "CLCProt", "auras")
AceConfigDialog:AddToBlizOptions("CLCProt", "Layout", "CLCProt", "layout")
	
-- profiles
options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(clcprot.db)
options.args.profiles.order = 900
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("CLCProt", "Profiles", "CLCProt", "profiles")

InterfaceOptionsFrame_OpenToCategory("CLCProt")

--- END OF FILE ---------------------------------------------------------------=